from typing import Final

DEBUG: Final[bool]
STATIC_DIR: Final[None]
CUT_TRACEBACK: Final[bool]
STD_DOCTYPE: Final[str]
STD_STYLESHEETS: Final[list[tuple[str, ...]]]
BASE_STYLESHEETS_PLACEHOLDER: Final[str]
COMPONENT_STYLESHEETS_PLACEHOLDER: Final[str]
SCRIPTS_PLACEHOLDER: Final[str]
RELOADING_CHECK_INTERVAL: Final[float]
LOG_TO_SQLITE: Final[None]
LOGGING_LEVEL: Final[None]
LOGGING_PONY_LEVEL: Final[None]
MAX_SESSION_CTIME: Final[int]
MAX_SESSION_MTIME: Final[int]
MAX_LONGLIFE_SESSION: Final[int]
COOKIE_SERIALIZATION_TYPE: Final[str]
COOKIE_NAME: Final[str]
COOKIE_PATH: Final[str]
COOKIE_DOMAIN: Final[None]
HASH_ALGORITHM: Final[None]
SESSION_STORAGE: Final[None]
MEMCACHE: Final[None]
ALTERNATIVE_SESSION_MEMCACHE: Final[None]
ALTERNATIVE_ORM_MEMCACHE: Final[None]
ALTERNATIVE_TEMPLATING_MEMCACHE: Final[None]
ALTERNATIVE_RESPONSE_MEMCACHE: Final[None]
PICKLE_START_OFFSET: Final[int]
PICKLE_HTML_AS_PLAIN_STR: Final[bool]
RESTORE_ESCAPES: Final[bool]
SOURCE_ENCODING: Final[None]
CONSOLE_ENCODING: Final[None]
MAX_FETCH_COUNT: Final[None]
CONSOLE_WIDTH: Final[int]
SIMPLE_ALIASES: Final[bool]
INNER_JOIN_SYNTAX: Final[bool]
DEBUGGING_REMOVE_ADDR: Final[bool]
DEBUGGING_RESTORE_ESCAPES: Final[bool]
