from typing import Final

XK_hebrew_doublelowline: Final = 0xCDF
XK_hebrew_aleph: Final = 0xCE0
XK_hebrew_bet: Final = 0xCE1
XK_hebrew_beth: Final = 0xCE1
XK_hebrew_gimel: Final = 0xCE2
XK_hebrew_gimmel: Final = 0xCE2
XK_hebrew_dalet: Final = 0xCE3
XK_hebrew_daleth: Final = 0xCE3
XK_hebrew_he: Final = 0xCE4
XK_hebrew_waw: Final = 0xCE5
XK_hebrew_zain: Final = 0xCE6
XK_hebrew_zayin: Final = 0xCE6
XK_hebrew_chet: Final = 0xCE7
XK_hebrew_het: Final = 0xCE7
XK_hebrew_tet: Final = 0xCE8
XK_hebrew_teth: Final = 0xCE8
XK_hebrew_yod: Final = 0xCE9
XK_hebrew_finalkaph: Final = 0xCEA
XK_hebrew_kaph: Final = 0xCEB
XK_hebrew_lamed: Final = 0xCEC
XK_hebrew_finalmem: Final = 0xCED
XK_hebrew_mem: Final = 0xCEE
XK_hebrew_finalnun: Final = 0xCEF
XK_hebrew_nun: Final = 0xCF0
XK_hebrew_samech: Final = 0xCF1
XK_hebrew_samekh: Final = 0xCF1
XK_hebrew_ayin: Final = 0xCF2
XK_hebrew_finalpe: Final = 0xCF3
XK_hebrew_pe: Final = 0xCF4
XK_hebrew_finalzade: Final = 0xCF5
XK_hebrew_finalzadi: Final = 0xCF5
XK_hebrew_zade: Final = 0xCF6
XK_hebrew_zadi: Final = 0xCF6
XK_hebrew_qoph: Final = 0xCF7
XK_hebrew_kuf: Final = 0xCF7
XK_hebrew_resh: Final = 0xCF8
XK_hebrew_shin: Final = 0xCF9
XK_hebrew_taw: Final = 0xCFA
XK_hebrew_taf: Final = 0xCFA
XK_Hebrew_switch: Final = 0xFF7E
