from typing import Final, Literal

WEEKDAYS: Final = ("Ravivāra", "Somavāra", "Maṅgalavāra", "Budhavāra", "Guruvāra", "Śukravāra", "Śanivāra")
MONTHS: Final[tuple[str, ...]]
HAVE_31_DAYS: Final = (2, 3, 4, 5, 6)
HAVE_30_DAYS: Final = (7, 8, 9, 10, 11, 12)
SAKA_EPOCH: Final = 78

def to_jd(year: int, month: int, day: int) -> float: ...
def from_jd(jd: float) -> tuple[int, int, int]: ...
def from_gregorian(year: int, month: int, day: int) -> tuple[int, int, int]: ...
def to_gregorian(year: int, month: int, day: int) -> tuple[int, int, int]: ...
def month_length(year: int, month: int) -> Literal[30, 31]: ...
def monthcalendar(year: int, month: int) -> list[list[int | None]]: ...
def format(year: int, month: int, day: int) -> str: ...
