import datetime

from ..core import WesternCalendar

class Barbados(WesternCalendar):
    non_computable_holiday_dict: dict[int, list[tuple[datetime.date, str]]]
    def get_kadooment_day(self, year: int) -> tuple[datetime.date, str]: ...
    def get_emancipation_day(self, year: int) -> tuple[datetime.date, str]: ...
    def non_computable_holiday(self, year: int) -> list[tuple[datetime.date, str]] | None: ...
    def get_fixed_holidays(self, year: int) -> list[tuple[datetime.date, str]]: ...
