from .australia import (
    Australia as Australia,
    AustralianCapitalTerritory as AustralianCapitalTerritory,
    Hobart as Hobart,
    NewSouthWales as NewSouthWales,
    NorthernTerritory as NorthernTerritory,
    Queensland as Queensland,
    SouthAustralia as SouthAustralia,
    Tasmania as Tasmania,
    Victoria as Victoria,
    WesternAustralia as WesternAustralia,
)
from .marshall_islands import MarshallIslands as MarshallIslands
from .new_zealand import NewZealand as NewZealand

__all__ = (
    # Australia and al.
    "Australia",
    "AustralianCapitalTerritory",
    "NewSouthWales",
    "NorthernTerritory",
    "Queensland",
    "SouthAustralia",
    "Tasmania",
    "Hobart",
    "Victoria",
    "WesternAustralia",
    # Other oceanian countries
    "MarshallIslands",
    "NewZealand",
)
