# Internal type that is commonly used as a base class
# and some public apis the signature needs it. As type
# is internal exact module it lives in is unstable across
# versions.
class Trackable: ...
