def to_jd(year: int, dayofyear: int) -> float: ...
def from_jd(jd: float) -> tuple[int, int]: ...
def from_gregorian(year: int, month: int, day: int) -> tuple[int, int]: ...
def to_gregorian(year: int, dayofyear: int) -> tuple[int, int, int]: ...
