from typing import Final

XK_leftcaret: Final = 0xBA3
XK_rightcaret: Final = 0xBA6
XK_downcaret: Final = 0xBA8
XK_upcaret: Final = 0xBA9
XK_overbar: Final = 0xBC0
XK_downtack: Final = 0xBC2
XK_upshoe: Final = 0xBC3
XK_downstile: Final = 0xBC4
XK_underbar: Final = 0xBC6
XK_jot: Final = 0xBCA
XK_quad: Final = 0xBCC
XK_uptack: Final = 0xBCE
XK_circle: Final = 0xBCF
XK_upstile: Final = 0xBD3
XK_downshoe: Final = 0xBD6
XK_rightshoe: Final = 0xBD8
XK_leftshoe: Final = 0xBDA
XK_lefttack: Final = 0xBDC
XK_righttack: Final = 0xBFC
