from authlib.jose.rfc7515 import JWSAlgorithm

class EdDSAAlgorithm(JWSAlgorithm):
    name: str
    description: str
    def prepare_key(self, raw_data): ...
    def sign(self, msg, key): ...
    def verify(self, msg, sig, key): ...

def register_jws_rfc8037(cls) -> None: ...
