import datetime

from ..core import WesternCalendar

class Sweden(WesternCalendar):
    def get_midsummer_eve(self, year: int) -> datetime.date: ...
    def get_midsummer_day(self, year: int) -> datetime.date: ...
    def get_variable_all_saints(self, year: int) -> datetime.date: ...
