def system_path_separator() -> str: ...
def XDG_DATA_HOME() -> str: ...
def XDG_CONFIG_HOME() -> str: ...
def XDG_CACHE_HOME() -> str: ...
def XDG_RUNTIME_DIR() -> str: ...
def XDG_DATA_DIRS() -> str: ...
def XDG_CONFIG_DIRS() -> str: ...
