from _typeshed import Incomplete

color01: Incomplete
color02: Incomplete
color03: Incomplete
color04: Incomplete
color05: Incomplete
color06: Incomplete
color07: Incomplete
color08: Incomplete
color09: Incomplete
color10: Incomplete
color01Light: Incomplete
color02Light: Incomplete
color03Light: Incomplete
color04Light: Incomplete
color05Light: Incomplete
color06Light: Incomplete
color07Light: Incomplete
color08Light: Incomplete
color09Light: Incomplete
color10Light: Incomplete
color01Dark: Incomplete
color02Dark: Incomplete
color03Dark: Incomplete
color04Dark: Incomplete
color05Dark: Incomplete
color06Dark: Incomplete
color07Dark: Incomplete
color08Dark: Incomplete
color09Dark: Incomplete
color10Dark: Incomplete
backgroundGrey: Incomplete
