from .base import AuthenticationBase

class Passwordless(AuthenticationBase):
    def email(self, email: str, send: str = "link", auth_params: dict[str, str] | None = None): ...
    def sms(self, phone_number: str): ...
