from typing import Final

XK_Arabic_comma: Final = 0x5AC
XK_Arabic_semicolon: Final = 0x5BB
XK_Arabic_question_mark: Final = 0x5BF
XK_Arabic_hamza: Final = 0x5C1
XK_Arabic_maddaonalef: Final = 0x5C2
XK_Arabic_hamzaonalef: Final = 0x5C3
XK_Arabic_hamzaonwaw: Final = 0x5C4
XK_Arabic_hamzaunderalef: Final = 0x5C5
XK_Arabic_hamzaonyeh: Final = 0x5C6
XK_Arabic_alef: Final = 0x5C7
XK_Arabic_beh: Final = 0x5C8
XK_Arabic_tehmarbuta: Final = 0x5C9
XK_Arabic_teh: Final = 0x5CA
XK_Arabic_theh: Final = 0x5CB
XK_Arabic_jeem: Final = 0x5CC
XK_Arabic_hah: Final = 0x5CD
XK_Arabic_khah: Final = 0x5CE
XK_Arabic_dal: Final = 0x5CF
XK_Arabic_thal: Final = 0x5D0
XK_Arabic_ra: Final = 0x5D1
XK_Arabic_zain: Final = 0x5D2
XK_Arabic_seen: Final = 0x5D3
XK_Arabic_sheen: Final = 0x5D4
XK_Arabic_sad: Final = 0x5D5
XK_Arabic_dad: Final = 0x5D6
XK_Arabic_tah: Final = 0x5D7
XK_Arabic_zah: Final = 0x5D8
XK_Arabic_ain: Final = 0x5D9
XK_Arabic_ghain: Final = 0x5DA
XK_Arabic_tatweel: Final = 0x5E0
XK_Arabic_feh: Final = 0x5E1
XK_Arabic_qaf: Final = 0x5E2
XK_Arabic_kaf: Final = 0x5E3
XK_Arabic_lam: Final = 0x5E4
XK_Arabic_meem: Final = 0x5E5
XK_Arabic_noon: Final = 0x5E6
XK_Arabic_ha: Final = 0x5E7
XK_Arabic_heh: Final = 0x5E7
XK_Arabic_waw: Final = 0x5E8
XK_Arabic_alefmaksura: Final = 0x5E9
XK_Arabic_yeh: Final = 0x5EA
XK_Arabic_fathatan: Final = 0x5EB
XK_Arabic_dammatan: Final = 0x5EC
XK_Arabic_kasratan: Final = 0x5ED
XK_Arabic_fatha: Final = 0x5EE
XK_Arabic_damma: Final = 0x5EF
XK_Arabic_kasra: Final = 0x5F0
XK_Arabic_shadda: Final = 0x5F1
XK_Arabic_sukun: Final = 0x5F2
XK_Arabic_switch: Final = 0xFF7E
