import datetime

from ..core import WesternCalendar

class Guernsey(WesternCalendar):
    def get_spring_bank_holiday(self, year: int) -> tuple[datetime.date, str]: ...
    def get_early_may_bank_holiday(self, year: int) -> tuple[datetime.date, str]: ...
    def get_summer_bank_holiday(self, year: int) -> tuple[datetime.date, str]: ...
    def get_liberation_day(self, year: int) -> tuple[datetime.date, str]: ...
