from authlib.oauth1.rfc5849.base_server import BaseServer

class ResourceProtector(BaseServer):
    def validate_request(self, method, uri, body, headers): ...
    def get_token_credential(self, request) -> None: ...
