from typing import Final

XK_XF86_ModeLock: Final = 0x1008FF01
XK_XF86_MonBrightnessUp: Final = 0x1008FF02
XK_XF86_MonBrightnessDown: Final = 0x1008FF03
XK_XF86_KbdLightOnOff: Final = 0x1008FF04
XK_XF86_KbdBrightnessUp: Final = 0x1008FF05
XK_XF86_KbdBrightnessDown: Final = 0x1008FF06
XK_XF86_MonBrightnessCycle: Final = 0x1008FF07
XK_XF86_Standby: Final = 0x1008FF10
XK_XF86_AudioLowerVolume: Final = 0x1008FF11
XK_XF86_AudioMute: Final = 0x1008FF12
XK_XF86_AudioRaiseVolume: Final = 0x1008FF13
XK_XF86_AudioPlay: Final = 0x1008FF14
XK_XF86_AudioStop: Final = 0x1008FF15
XK_XF86_AudioPrev: Final = 0x1008FF16
XK_XF86_AudioNext: Final = 0x1008FF17
XK_XF86_HomePage: Final = 0x1008FF18
XK_XF86_Mail: Final = 0x1008FF19
XK_XF86_Start: Final = 0x1008FF1A
XK_XF86_Search: Final = 0x1008FF1B
XK_XF86_AudioRecord: Final = 0x1008FF1C
XK_XF86_Calculator: Final = 0x1008FF1D
XK_XF86_Memo: Final = 0x1008FF1E
XK_XF86_ToDoList: Final = 0x1008FF1F
XK_XF86_Calendar: Final = 0x1008FF20
XK_XF86_PowerDown: Final = 0x1008FF21
XK_XF86_ContrastAdjust: Final = 0x1008FF22
XK_XF86_RockerUp: Final = 0x1008FF23
XK_XF86_RockerDown: Final = 0x1008FF24
XK_XF86_RockerEnter: Final = 0x1008FF25
XK_XF86_Back: Final = 0x1008FF26
XK_XF86_Forward: Final = 0x1008FF27
XK_XF86_Stop: Final = 0x1008FF28
XK_XF86_Refresh: Final = 0x1008FF29
XK_XF86_PowerOff: Final = 0x1008FF2A
XK_XF86_WakeUp: Final = 0x1008FF2B
XK_XF86_Eject: Final = 0x1008FF2C
XK_XF86_ScreenSaver: Final = 0x1008FF2D
XK_XF86_WWW: Final = 0x1008FF2E
XK_XF86_Sleep: Final = 0x1008FF2F
XK_XF86_Favorites: Final = 0x1008FF30
XK_XF86_AudioPause: Final = 0x1008FF31
XK_XF86_AudioMedia: Final = 0x1008FF32
XK_XF86_MyComputer: Final = 0x1008FF33
XK_XF86_VendorHome: Final = 0x1008FF34
XK_XF86_LightBulb: Final = 0x1008FF35
XK_XF86_Shop: Final = 0x1008FF36
XK_XF86_History: Final = 0x1008FF37
XK_XF86_OpenURL: Final = 0x1008FF38
XK_XF86_AddFavorite: Final = 0x1008FF39
XK_XF86_HotLinks: Final = 0x1008FF3A
XK_XF86_BrightnessAdjust: Final = 0x1008FF3B
XK_XF86_Finance: Final = 0x1008FF3C
XK_XF86_Community: Final = 0x1008FF3D
XK_XF86_AudioRewind: Final = 0x1008FF3E
XK_XF86_XF86BackForward: Final = 0x1008FF3F
XK_XF86_Launch0: Final = 0x1008FF40
XK_XF86_Launch1: Final = 0x1008FF41
XK_XF86_Launch2: Final = 0x1008FF42
XK_XF86_Launch3: Final = 0x1008FF43
XK_XF86_Launch4: Final = 0x1008FF44
XK_XF86_Launch5: Final = 0x1008FF45
XK_XF86_Launch6: Final = 0x1008FF46
XK_XF86_Launch7: Final = 0x1008FF47
XK_XF86_Launch8: Final = 0x1008FF48
XK_XF86_Launch9: Final = 0x1008FF49
XK_XF86_LaunchA: Final = 0x1008FF4A
XK_XF86_LaunchB: Final = 0x1008FF4B
XK_XF86_LaunchC: Final = 0x1008FF4C
XK_XF86_LaunchD: Final = 0x1008FF4D
XK_XF86_LaunchE: Final = 0x1008FF4E
XK_XF86_LaunchF: Final = 0x1008FF4F
XK_XF86_ApplicationLeft: Final = 0x1008FF50
XK_XF86_ApplicationRight: Final = 0x1008FF51
XK_XF86_Book: Final = 0x1008FF52
XK_XF86_CD: Final = 0x1008FF53
XK_XF86_Calculater: Final = 0x1008FF54
XK_XF86_Clear: Final = 0x1008FF55
XK_XF86_Close: Final = 0x1008FF56
XK_XF86_Copy: Final = 0x1008FF57
XK_XF86_Cut: Final = 0x1008FF58
XK_XF86_Display: Final = 0x1008FF59
XK_XF86_DOS: Final = 0x1008FF5A
XK_XF86_Documents: Final = 0x1008FF5B
XK_XF86_Excel: Final = 0x1008FF5C
XK_XF86_Explorer: Final = 0x1008FF5D
XK_XF86_Game: Final = 0x1008FF5E
XK_XF86_Go: Final = 0x1008FF5F
XK_XF86_iTouch: Final = 0x1008FF60
XK_XF86_LogOff: Final = 0x1008FF61
XK_XF86_Market: Final = 0x1008FF62
XK_XF86_Meeting: Final = 0x1008FF63
XK_XF86_MenuKB: Final = 0x1008FF65
XK_XF86_MenuPB: Final = 0x1008FF66
XK_XF86_MySites: Final = 0x1008FF67
XK_XF86_New: Final = 0x1008FF68
XK_XF86_News: Final = 0x1008FF69
XK_XF86_OfficeHome: Final = 0x1008FF6A
XK_XF86_Open: Final = 0x1008FF6B
XK_XF86_Option: Final = 0x1008FF6C
XK_XF86_Paste: Final = 0x1008FF6D
XK_XF86_Phone: Final = 0x1008FF6E
XK_XF86_Q: Final = 0x1008FF70
XK_XF86_Reply: Final = 0x1008FF72
XK_XF86_Reload: Final = 0x1008FF73
XK_XF86_RotateWindows: Final = 0x1008FF74
XK_XF86_RotationPB: Final = 0x1008FF75
XK_XF86_RotationKB: Final = 0x1008FF76
XK_XF86_Save: Final = 0x1008FF77
XK_XF86_ScrollUp: Final = 0x1008FF78
XK_XF86_ScrollDown: Final = 0x1008FF79
XK_XF86_ScrollClick: Final = 0x1008FF7A
XK_XF86_Send: Final = 0x1008FF7B
XK_XF86_Spell: Final = 0x1008FF7C
XK_XF86_SplitScreen: Final = 0x1008FF7D
XK_XF86_Support: Final = 0x1008FF7E
XK_XF86_TaskPane: Final = 0x1008FF7F
XK_XF86_Terminal: Final = 0x1008FF80
XK_XF86_Tools: Final = 0x1008FF81
XK_XF86_Travel: Final = 0x1008FF82
XK_XF86_UserPB: Final = 0x1008FF84
XK_XF86_User1KB: Final = 0x1008FF85
XK_XF86_User2KB: Final = 0x1008FF86
XK_XF86_Video: Final = 0x1008FF87
XK_XF86_WheelButton: Final = 0x1008FF88
XK_XF86_Word: Final = 0x1008FF89
XK_XF86_Xfer: Final = 0x1008FF8A
XK_XF86_ZoomIn: Final = 0x1008FF8B
XK_XF86_ZoomOut: Final = 0x1008FF8C
XK_XF86_Away: Final = 0x1008FF8D
XK_XF86_Messenger: Final = 0x1008FF8E
XK_XF86_WebCam: Final = 0x1008FF8F
XK_XF86_MailForward: Final = 0x1008FF90
XK_XF86_Pictures: Final = 0x1008FF91
XK_XF86_Music: Final = 0x1008FF92
XK_XF86_Battery: Final = 0x1008FF93
XK_XF86_Bluetooth: Final = 0x1008FF94
XK_XF86_WLAN: Final = 0x1008FF95
XK_XF86_UWB: Final = 0x1008FF96
XK_XF86_AudioForward: Final = 0x1008FF97
XK_XF86_AudioRepeat: Final = 0x1008FF98
XK_XF86_AudioRandomPlay: Final = 0x1008FF99
XK_XF86_Subtitle: Final = 0x1008FF9A
XK_XF86_AudioCycleTrack: Final = 0x1008FF9B
XK_XF86_CycleAngle: Final = 0x1008FF9C
XK_XF86_FrameBack: Final = 0x1008FF9D
XK_XF86_FrameForward: Final = 0x1008FF9E
XK_XF86_Time: Final = 0x1008FF9F
XK_XF86_Select: Final = 0x1008FFA0
XK_XF86_View: Final = 0x1008FFA1
XK_XF86_TopMenu: Final = 0x1008FFA2
XK_XF86_Red: Final = 0x1008FFA3
XK_XF86_Green: Final = 0x1008FFA4
XK_XF86_Yellow: Final = 0x1008FFA5
XK_XF86_Blue: Final = 0x1008FFA6
XK_XF86_Suspend: Final = 0x1008FFA7
XK_XF86_Hibernate: Final = 0x1008FFA8
XK_XF86_TouchpadToggle: Final = 0x1008FFA9
XK_XF86_TouchpadOn: Final = 0x1008FFB0
XK_XF86_TouchpadOff: Final = 0x1008FFB1
XK_XF86_AudioMicMute: Final = 0x1008FFB2
XK_XF86_Keyboard: Final = 0x1008FFB3
XK_XF86_WWAN: Final = 0x1008FFB4
XK_XF86_RFKill: Final = 0x1008FFB5
XK_XF86_AudioPreset: Final = 0x1008FFB6
XK_XF86_RotationLockToggle: Final = 0x1008FFB7
XK_XF86_FullScreen: Final = 0x1008FFB8
XK_XF86_Switch_VT_1: Final = 0x1008FE01
XK_XF86_Switch_VT_2: Final = 0x1008FE02
XK_XF86_Switch_VT_3: Final = 0x1008FE03
XK_XF86_Switch_VT_4: Final = 0x1008FE04
XK_XF86_Switch_VT_5: Final = 0x1008FE05
XK_XF86_Switch_VT_6: Final = 0x1008FE06
XK_XF86_Switch_VT_7: Final = 0x1008FE07
XK_XF86_Switch_VT_8: Final = 0x1008FE08
XK_XF86_Switch_VT_9: Final = 0x1008FE09
XK_XF86_Switch_VT_10: Final = 0x1008FE0A
XK_XF86_Switch_VT_11: Final = 0x1008FE0B
XK_XF86_Switch_VT_12: Final = 0x1008FE0C
XK_XF86_Ungrab: Final = 0x1008FE20
XK_XF86_ClearGrab: Final = 0x1008FE21
XK_XF86_Next_VMode: Final = 0x1008FE22
XK_XF86_Prev_VMode: Final = 0x1008FE23
XK_XF86_LogWindowTree: Final = 0x1008FE24
XK_XF86_LogGrabInfo: Final = 0x1008FE25
