class XrayCommandListener:
    record_full_documents: bool
    def __init__(self, record_full_documents: bool) -> None: ...
    def started(self, event) -> None: ...
    def succeeded(self, event) -> None: ...
    def failed(self, event) -> None: ...

def patch(record_full_documents: bool = False) -> None: ...
