from hvac.api.system_backend.system_backend_mixin import SystemBackendMixin
from requests.models import _JSON

class Wrapping(SystemBackendMixin):
    def unwrap(self, token=None): ...
    def wrap(self, payload: _JSON | None = None, ttl: int = 60): ...
