import ast

from flake8_simplify.utils import UnaryOp

def get_sim201(node: UnaryOp) -> list[tuple[int, int, str]]: ...
def get_sim202(node: UnaryOp) -> list[tuple[int, int, str]]: ...
def get_sim203(node: UnaryOp) -> list[tuple[int, int, str]]: ...
def get_sim208(node: ast.UnaryOp) -> list[tuple[int, int, str]]: ...
