import ast

from flake8_simplify.utils import For

def get_sim104(node: ast.For) -> list[tuple[int, int, str]]: ...
def get_sim110_sim111(node: ast.For) -> list[tuple[int, int, str]]: ...
def get_sim113(node: For) -> list[tuple[int, int, str]]: ...
