import datetime
from typing import ClassVar

from ..core import WesternCalendar

class Argentina(WesternCalendar):
    fat_tuesday_label: ClassVar[str]
    def get_general_guemes_day(self, year: int) -> tuple[datetime.date, str]: ...
    def get_general_martin_day(self, year: int) -> tuple[datetime.date, str]: ...
    def get_soberania_day(self, year: int) -> tuple[datetime.date, str]: ...
    def get_diversidad_day(self, year: int) -> tuple[datetime.date, str]: ...
    def get_malvinas_day(self, year: int) -> tuple[datetime.date, str]: ...
