from .argentina import Argentina as Argentina
from .barbados import Barbados as Barbados
from .brazil import (
    Brazil as Brazil,
    BrazilAcre as BrazilAcre,
    BrazilAlagoas as BrazilAlagoas,
    BrazilAmapa as BrazilAmapa,
    BrazilAmazonas as BrazilAmazonas,
    BrazilAracajuCity as BrazilAracajuCity,
    BrazilBahia as BrazilBahia,
    BrazilBankCalendar as BrazilBankCalendar,
    BrazilBelemCity as BrazilBelemCity,
    BrazilBeloHorizonteCity as BrazilBeloHorizonteCity,
    BrazilBoaVistaCity as BrazilBoaVistaCity,
    BrazilCampoGrandeCity as BrazilCampoGrandeCity,
    BrazilCariacicaCity as BrazilCariacicaCity,
    BrazilCeara as BrazilCeara,
    BrazilChapecoCity as BrazilChapecoCity,
    BrazilCuiabaCity as BrazilCuiabaCity,
    BrazilCuritibaCity as BrazilCuritibaCity,
    BrazilDistritoFederal as BrazilDistritoFederal,
    BrazilEspiritoSanto as BrazilEspiritoSanto,
    BrazilFlorianopolisCity as BrazilFlorianopolisCity,
    BrazilFortalezaCity as BrazilFortalezaCity,
    BrazilGoianiaCity as BrazilGoianiaCity,
    BrazilGoias as BrazilGoias,
    BrazilGuarapariCity as BrazilGuarapariCity,
    BrazilJoaoPessoaCity as BrazilJoaoPessoaCity,
    BrazilJoinvilleCity as BrazilJoinvilleCity,
    BrazilMacapaCity as BrazilMacapaCity,
    BrazilMaceioCity as BrazilMaceioCity,
    BrazilManausCity as BrazilManausCity,
    BrazilMaranhao as BrazilMaranhao,
    BrazilMatoGrosso as BrazilMatoGrosso,
    BrazilMatoGrossoDoSul as BrazilMatoGrossoDoSul,
    BrazilMinasGerais as BrazilMinasGerais,
    BrazilNatalCity as BrazilNatalCity,
    BrazilPalmasCity as BrazilPalmasCity,
    BrazilPara as BrazilPara,
    BrazilParaiba as BrazilParaiba,
    BrazilParana as BrazilParana,
    BrazilPernambuco as BrazilPernambuco,
    BrazilPiaui as BrazilPiaui,
    BrazilPortoAlegreCity as BrazilPortoAlegreCity,
    BrazilPortoVelhoCity as BrazilPortoVelhoCity,
    BrazilRecifeCity as BrazilRecifeCity,
    BrazilRioBrancoCity as BrazilRioBrancoCity,
    BrazilRioDeJaneiro as BrazilRioDeJaneiro,
    BrazilRioGrandeDoNorte as BrazilRioGrandeDoNorte,
    BrazilRioGrandeDoSul as BrazilRioGrandeDoSul,
    BrazilRondonia as BrazilRondonia,
    BrazilRoraima as BrazilRoraima,
    BrazilSalvadorCity as BrazilSalvadorCity,
    BrazilSantaCatarina as BrazilSantaCatarina,
    BrazilSaoPauloCity as BrazilSaoPauloCity,
    BrazilSaoPauloState as BrazilSaoPauloState,
    BrazilSergipe as BrazilSergipe,
    BrazilSerraCity as BrazilSerraCity,
    BrazilSorocabaCity as BrazilSorocabaCity,
    BrazilTeresinaCity as BrazilTeresinaCity,
    BrazilTocantins as BrazilTocantins,
    BrazilVilaVelhaCity as BrazilVilaVelhaCity,
    BrazilVitoriaCity as BrazilVitoriaCity,
)
from .canada import (
    Alberta as Alberta,
    BritishColumbia as BritishColumbia,
    Canada as Canada,
    Manitoba as Manitoba,
    NewBrunswick as NewBrunswick,
    Newfoundland as Newfoundland,
    NorthwestTerritories as NorthwestTerritories,
    NovaScotia as NovaScotia,
    Nunavut as Nunavut,
    Ontario as Ontario,
    PrinceEdwardIsland as PrinceEdwardIsland,
    Quebec as Quebec,
    Saskatchewan as Saskatchewan,
    Yukon as Yukon,
)
from .chile import Chile as Chile
from .colombia import Colombia as Colombia
from .el_salvador import ElSalvador as ElSalvador
from .mexico import Mexico as Mexico
from .panama import Panama as Panama
from .paraguay import Paraguay as Paraguay

__all__ = (
    # Brazil & its states.
    "Brazil",
    "BrazilAcre",
    "BrazilAlagoas",
    "BrazilAmapa",
    "BrazilAmazonas",
    "BrazilBahia",
    "BrazilCeara",
    "BrazilDistritoFederal",
    "BrazilEspiritoSanto",
    "BrazilGoias",
    "BrazilMaranhao",
    "BrazilMinasGerais",
    "BrazilMatoGrosso",
    "BrazilMatoGrossoDoSul",
    "BrazilPara",
    "BrazilParaiba",
    "BrazilPernambuco",
    "BrazilPiaui",
    "BrazilParana",
    "BrazilRioDeJaneiro",
    "BrazilRioGrandeDoNorte",
    "BrazilRioGrandeDoSul",
    "BrazilRondonia",
    "BrazilRoraima",
    "BrazilSantaCatarina",
    "BrazilSaoPauloState",
    "BrazilSaoPauloCity",
    "BrazilSergipe",
    "BrazilTocantins",
    "BrazilVitoriaCity",
    "BrazilVilaVelhaCity",
    "BrazilCariacicaCity",
    "BrazilGuarapariCity",
    "BrazilSerraCity",
    "BrazilBankCalendar",
    "BrazilRioBrancoCity",
    "BrazilMaceioCity",
    "BrazilManausCity",
    "BrazilMacapaCity",
    "BrazilSalvadorCity",
    "BrazilFortalezaCity",
    "BrazilGoianiaCity",
    "BrazilBeloHorizonteCity",
    "BrazilCampoGrandeCity",
    "BrazilCuiabaCity",
    "BrazilBelemCity",
    "BrazilJoaoPessoaCity",
    "BrazilRecifeCity",
    "BrazilTeresinaCity",
    "BrazilCuritibaCity",
    "BrazilNatalCity",
    "BrazilPortoVelhoCity",
    "BrazilBoaVistaCity",
    "BrazilPortoAlegreCity",
    "BrazilChapecoCity",
    "BrazilFlorianopolisCity",
    "BrazilJoinvilleCity",
    "BrazilAracajuCity",
    "BrazilSorocabaCity",
    "BrazilPalmasCity",
    # Canada
    "Canada",
    "Ontario",
    "Quebec",
    "BritishColumbia",
    "Alberta",
    "Saskatchewan",
    "Manitoba",
    "NewBrunswick",
    "NovaScotia",
    "PrinceEdwardIsland",
    "Newfoundland",
    "Yukon",
    "NorthwestTerritories",
    "Nunavut",
    # Other american countries
    "Barbados",
    "Chile",
    "Colombia",
    "Mexico",
    "Panama",
    "Paraguay",
    "Argentina",
    "ElSalvador",
)
