from typing import Final

__version__: Final[str]

def makeEmptySquare(x, y, size, color): ...
def makeFilledSquare(x, y, size, color): ...
def makeFilledDiamond(x, y, size, color): ...
def makeEmptyCircle(x, y, size, color): ...
def makeFilledCircle(x, y, size, color): ...
def makeSmiley(x, y, size, color): ...
