import datetime
from typing import Final

EPOCH: Final = 2415020

def to_gregorian(dc: int) -> tuple[int, int, int]: ...
def from_gregorian(year: int, month: int, day: int) -> float: ...
def to_jd(dc: int) -> float: ...
def from_jd(jdc: float) -> float: ...
def from_julian(year: int, month: int, day: int) -> float: ...
def to_julian(dc: int) -> tuple[int, int, int]: ...
def to_datetime(dc: int) -> datetime.datetime: ...
def from_datetime(date: datetime.datetime) -> float: ...
