import datetime

class DayCount:
    epoch: float
    def __init__(self, epoch: float) -> None: ...
    def to_gregorian(self, dc: int) -> tuple[int, int, int]: ...
    def from_gregorian(self, year: int, month: int, day: int) -> float: ...
    def to_jd(self, dc: int) -> float: ...
    def from_jd(self, jdc: float) -> float: ...
    def from_julian(self, year: int, month: int, day: int) -> float: ...
    def to_julian(self, dc: int) -> tuple[int, int, int]: ...
    def to_datetime(self, dc: int) -> datetime.datetime: ...
    def from_datetime(self, date: datetime.datetime) -> float: ...
