from typing import Literal, overload

@overload
def str2bool(value: str, raise_exc: Literal[True]) -> bool: ...
@overload
def str2bool(value: str, raise_exc: bool = False) -> bool | None: ...
def str2bool_exc(value: str) -> bool: ...
