from typing import Final

REMOTE_NAMESPACE: Final = "remote"
LOCAL_NAMESPACE: Final = "local"
LOCAL_EXCEPTIONS: tuple[type[Exception], ...]

async def begin_subsegment(session, trace_config_ctx, params): ...
async def end_subsegment(session, trace_config_ctx, params): ...
async def end_subsegment_with_exception(session, trace_config_ctx, params): ...
def aws_xray_trace_config(name=None): ...
