import datetime
from typing import Final, Literal

MON: Final = 0
TUE: Final = 1
WED: Final = 2
THU: Final = 3
FRI: Final = 4
SAT: Final = 5
SUN: Final = 6

def to_jd(year: int, week: int, day: int) -> float: ...
def from_jd(jd: float) -> datetime._IsoCalendarDate: ...
def weeks_per_year(year: int) -> Literal[52, 53]: ...
def from_gregorian(year: int, month: int, day: int) -> datetime._IsoCalendarDate: ...
def to_gregorian(year: int, week: int, day: int) -> tuple[int, int, int]: ...
def format(year: int, week: int, day: int) -> str: ...
