from hvac.api.system_backend.system_backend_mixin import SystemBackendMixin

class Quota(SystemBackendMixin):
    def read_quota(self, name): ...
    def list_quotas(self): ...
    def create_or_update_quota(
        self, name, rate, path=None, interval=None, block_interval=None, role=None, rate_limit_type=None, inheritable=None
    ): ...
    def delete_quota(self, name): ...
