def compare_operation(dn, attribute, value, auto_encode, schema=None, validator=None, check_names: bool = False): ...
def compare_request_to_dict(request): ...
def compare_response_to_dict(response): ...
