from .common import InfoExtractor

def gen_extractor_classes() -> list[type[InfoExtractor]]: ...
def gen_extractors() -> list[InfoExtractor]: ...
def list_extractor_classes(age_limit: int | None = None) -> list[type[InfoExtractor]]: ...
def list_extractors(age_limit: int | None = None) -> list[InfoExtractor]: ...
def get_info_extractor(ie_name: str) -> InfoExtractor: ...
def import_extractors() -> None: ...
