from typing import Final

XK_3270_Duplicate: Final = 0xFD01
XK_3270_FieldMark: Final = 0xFD02
XK_3270_Right2: Final = 0xFD03
XK_3270_Left2: Final = 0xFD04
XK_3270_BackTab: Final = 0xFD05
XK_3270_EraseEOF: Final = 0xFD06
XK_3270_EraseInput: Final = 0xFD07
XK_3270_Reset: Final = 0xFD08
XK_3270_Quit: Final = 0xFD09
XK_3270_PA1: Final = 0xFD0A
XK_3270_PA2: Final = 0xFD0B
XK_3270_PA3: Final = 0xFD0C
XK_3270_Test: Final = 0xFD0D
XK_3270_Attn: Final = 0xFD0E
XK_3270_CursorBlink: Final = 0xFD0F
XK_3270_AltCursor: Final = 0xFD10
XK_3270_KeyClick: Final = 0xFD11
XK_3270_Jump: Final = 0xFD12
XK_3270_Ident: Final = 0xFD13
XK_3270_Rule: Final = 0xFD14
XK_3270_Copy: Final = 0xFD15
XK_3270_Play: Final = 0xFD16
XK_3270_Setup: Final = 0xFD17
XK_3270_Record: Final = 0xFD18
XK_3270_ChangeScreen: Final = 0xFD19
XK_3270_DeleteWord: Final = 0xFD1A
XK_3270_ExSelect: Final = 0xFD1B
XK_3270_CursorSelect: Final = 0xFD1C
XK_3270_PrintScreen: Final = 0xFD1D
XK_3270_Enter: Final = 0xFD1E
