class md4:
    name: str
    digest_size: int
    digestsize: int
    block_size: int
    def __init__(self, content=None) -> None: ...
    def update(self, content) -> None: ...
    def copy(self): ...
    def digest(self): ...
    def hexdigest(self): ...

__all__ = ["md4"]
