from typing import Any

STATUS_INIT: str
STATUS_VIRTUAL: str
STATUS_MANDATORY_MISSING: str
STATUS_READ: str
STATUS_WRITABLE: str
STATUS_PENDING_CHANGES: str
STATUS_COMMITTED: str
STATUS_READY_FOR_DELETION: str
STATUS_READY_FOR_MOVING: str
STATUS_READY_FOR_RENAMING: str
STATUS_DELETED: str
STATUSES: Any
INITIAL_STATUSES: Any
