from _typeshed import Incomplete
from abc import ABCMeta
from collections.abc import Mapping

class Trie(Mapping[Incomplete, Incomplete], metaclass=ABCMeta):
    def keys(self, prefix=None): ...
    def has_keys_with_prefix(self, prefix): ...
    def longest_prefix(self, prefix): ...
    def longest_prefix_item(self, prefix): ...
