"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import google.protobuf.descriptor
from tensorflow.compiler.xla.tsl.protobuf.bfc_memory_map_pb2 import (
    BinSummary as BinSummary,
    MemAllocatorStats as MemAllocatorStats,
    MemChunk as MemChunk,
    MemoryDump as MemoryDump,
    SnapShot as SnapShot,
)

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor
