from _typeshed import StrOrBytesPath

def print_as_hex(s: str) -> None: ...
def get_starting_chunk(filename: StrOrBytesPath, length: int = 1024) -> bytes: ...
def is_binary_string(bytes_to_check: bytes | bytearray) -> bool: ...
