from typing import Any

def varlist(name, count): ...
def indent_block(block, padding): ...

BFSTR: Any

def render_encipher(write, indent: int = 0) -> None: ...
def write_encipher_function(write, indent: int = 0) -> None: ...
def write_expand_function(write, indent: int = 0) -> None: ...
def main() -> None: ...
