from typing import TypedDict, type_check_only
from typing_extensions import Self

@type_check_only
class _ParamsDict(TypedDict, total=False):
    businessName: str
    returnUrl: str
    cancelUrl: str

class BankAccountInstantVerificationJwtRequest:
    def __init__(self) -> None: ...
    def business_name(self, business_name: str) -> Self: ...
    def return_url(self, return_url: str) -> Self: ...
    def cancel_url(self, cancel_url: str) -> Self: ...
    def get_business_name(self) -> str: ...
    def get_return_url(self) -> str: ...
    def get_cancel_url(self) -> str: ...
    def to_graphql_variables(self) -> _ParamsDict: ...
