mathaccent: dict[str, str]
mathalpha: dict[str, str]
mathbin: dict[str, str]
mathclose: dict[str, str]
mathfence: dict[str, str]
mathop: dict[str, str]
mathopen: dict[str, str]
mathord: dict[str, str]
mathover: dict[str, str]
mathpunct: dict[str, str]
mathradical: dict[str, str]
mathrel: dict[str, str]
mathunder: dict[str, str]
space: dict[str, str]
