from typing import Any, Final
from typing_extensions import Self

__version__: Final[str]

class ABag:
    def __init__(self, **attr: Any) -> None: ...
    def clone(self, **attr: Any) -> Self: ...
    # ABag can have arbitrary attributes
    def __getattr__(self, name: str) -> Any: ...
    def __setattr__(self, name: str, value: Any) -> None: ...
