class IssuerParameter:
    def __call__(self, authorization_server) -> None: ...
    def add_issuer_parameter(self, authorization_server, response) -> None: ...
    def get_issuer(self) -> str | None: ...
