from typing import ClassVar

from .core import UnitedStates

class California(UnitedStates):
    shift_exceptions: ClassVar[list[tuple[int, int]]]

class CaliforniaEducation(California): ...
class CaliforniaBerkeley(California): ...
class CaliforniaSanFrancisco(California): ...
class CaliforniaWestHollywood(California): ...
