from _typeshed import Incomplete
from collections.abc import Callable
from types import ModuleType

def getDomBuilder(DomImplementation) -> dict[str, Incomplete]: ...

getDomModule: Callable[..., ModuleType]
