from _typeshed import Incomplete
from collections.abc import Generator
from re import Pattern

RESERVED_NS: Pattern[str]

def post_order_traverse(node) -> Generator[Incomplete, Incomplete]: ...
def reverse_post_order_traverse(node) -> Generator[Incomplete, Incomplete]: ...
def breadth_first_traverse(node) -> Generator[Incomplete]: ...
def longest_common_subsequence(left_sequence, right_sequence, eqfn=...): ...

WHITESPACE: Incomplete

def cleanup_whitespace(text): ...
def getpath(element, tree=None): ...
def make_ascii_tree(element): ...
