from typing import Final

XK_Thai_kokai: Final = 0xDA1
XK_Thai_khokhai: Final = 0xDA2
XK_Thai_khokhuat: Final = 0xDA3
XK_Thai_khokhwai: Final = 0xDA4
XK_Thai_khokhon: Final = 0xDA5
XK_Thai_khorakhang: Final = 0xDA6
XK_Thai_ngongu: Final = 0xDA7
XK_Thai_chochan: Final = 0xDA8
XK_Thai_choching: Final = 0xDA9
XK_Thai_chochang: Final = 0xDAA
XK_Thai_soso: Final = 0xDAB
XK_Thai_chochoe: Final = 0xDAC
XK_Thai_yoying: Final = 0xDAD
XK_Thai_dochada: Final = 0xDAE
XK_Thai_topatak: Final = 0xDAF
XK_Thai_thothan: Final = 0xDB0
XK_Thai_thonangmontho: Final = 0xDB1
XK_Thai_thophuthao: Final = 0xDB2
XK_Thai_nonen: Final = 0xDB3
XK_Thai_dodek: Final = 0xDB4
XK_Thai_totao: Final = 0xDB5
XK_Thai_thothung: Final = 0xDB6
XK_Thai_thothahan: Final = 0xDB7
XK_Thai_thothong: Final = 0xDB8
XK_Thai_nonu: Final = 0xDB9
XK_Thai_bobaimai: Final = 0xDBA
XK_Thai_popla: Final = 0xDBB
XK_Thai_phophung: Final = 0xDBC
XK_Thai_fofa: Final = 0xDBD
XK_Thai_phophan: Final = 0xDBE
XK_Thai_fofan: Final = 0xDBF
XK_Thai_phosamphao: Final = 0xDC0
XK_Thai_moma: Final = 0xDC1
XK_Thai_yoyak: Final = 0xDC2
XK_Thai_rorua: Final = 0xDC3
XK_Thai_ru: Final = 0xDC4
XK_Thai_loling: Final = 0xDC5
XK_Thai_lu: Final = 0xDC6
XK_Thai_wowaen: Final = 0xDC7
XK_Thai_sosala: Final = 0xDC8
XK_Thai_sorusi: Final = 0xDC9
XK_Thai_sosua: Final = 0xDCA
XK_Thai_hohip: Final = 0xDCB
XK_Thai_lochula: Final = 0xDCC
XK_Thai_oang: Final = 0xDCD
XK_Thai_honokhuk: Final = 0xDCE
XK_Thai_paiyannoi: Final = 0xDCF
XK_Thai_saraa: Final = 0xDD0
XK_Thai_maihanakat: Final = 0xDD1
XK_Thai_saraaa: Final = 0xDD2
XK_Thai_saraam: Final = 0xDD3
XK_Thai_sarai: Final = 0xDD4
XK_Thai_saraii: Final = 0xDD5
XK_Thai_saraue: Final = 0xDD6
XK_Thai_sarauee: Final = 0xDD7
XK_Thai_sarau: Final = 0xDD8
XK_Thai_sarauu: Final = 0xDD9
XK_Thai_phinthu: Final = 0xDDA
XK_Thai_maihanakat_maitho: Final = 0xDDE
XK_Thai_baht: Final = 0xDDF
XK_Thai_sarae: Final = 0xDE0
XK_Thai_saraae: Final = 0xDE1
XK_Thai_sarao: Final = 0xDE2
XK_Thai_saraaimaimuan: Final = 0xDE3
XK_Thai_saraaimaimalai: Final = 0xDE4
XK_Thai_lakkhangyao: Final = 0xDE5
XK_Thai_maiyamok: Final = 0xDE6
XK_Thai_maitaikhu: Final = 0xDE7
XK_Thai_maiek: Final = 0xDE8
XK_Thai_maitho: Final = 0xDE9
XK_Thai_maitri: Final = 0xDEA
XK_Thai_maichattawa: Final = 0xDEB
XK_Thai_thanthakhat: Final = 0xDEC
XK_Thai_nikhahit: Final = 0xDED
XK_Thai_leksun: Final = 0xDF0
XK_Thai_leknung: Final = 0xDF1
XK_Thai_leksong: Final = 0xDF2
XK_Thai_leksam: Final = 0xDF3
XK_Thai_leksi: Final = 0xDF4
XK_Thai_lekha: Final = 0xDF5
XK_Thai_lekhok: Final = 0xDF6
XK_Thai_lekchet: Final = 0xDF7
XK_Thai_lekpaet: Final = 0xDF8
XK_Thai_lekkao: Final = 0xDF9
