from typing import Final, Literal

TOL: Final = 1e-10

def machine_accuracy() -> tuple[float, int]: ...
def get_ordinal_suffix(ordinal: float) -> Literal["st", "nd", "rd", "th"]: ...
def iint(number: float) -> int: ...
def main() -> None: ...
