prTable: dict[int, str]

def GetPropTagName(pt): ...

mapiErrorTable: dict[int, str]

def GetScodeString(hr): ...

ptTable: dict[int, str]

def GetMapiTypeName(propType, rawType: bool = ...): ...
def GetProperties(obj, propList): ...
def GetAllProperties(obj, make_tag_names: bool = ...): ...
def SetPropertyValue(obj, prop, val) -> None: ...
def SetProperties(msg, propDict) -> None: ...
