SPAN_KIND: str
SPAN_KIND_RPC_CLIENT: str
SPAN_KIND_RPC_SERVER: str
SPAN_KIND_CONSUMER: str
SPAN_KIND_PRODUCER: str
SERVICE: str
ERROR: str
COMPONENT: str
SAMPLING_PRIORITY: str
PEER_SERVICE: str
PEER_HOSTNAME: str
PEER_ADDRESS: str
PEER_HOST_IPV4: str
PEER_HOST_IPV6: str
PEER_PORT: str
HTTP_URL: str
HTTP_METHOD: str
HTTP_STATUS_CODE: str
DATABASE_INSTANCE: str
DATABASE_STATEMENT: str
DATABASE_TYPE: str
DATABASE_USER: str
MESSAGE_BUS_DESTINATION: str
