from typing import Final

XK_kra: Final = 0x3A2
XK_kappa: Final = 0x3A2
XK_Rcedilla: Final = 0x3A3
XK_Itilde: Final = 0x3A5
XK_Lcedilla: Final = 0x3A6
XK_Emacron: Final = 0x3AA
XK_Gcedilla: Final = 0x3AB
XK_Tslash: Final = 0x3AC
XK_rcedilla: Final = 0x3B3
XK_itilde: Final = 0x3B5
XK_lcedilla: Final = 0x3B6
XK_emacron: Final = 0x3BA
XK_gcedilla: Final = 0x3BB
XK_tslash: Final = 0x3BC
XK_ENG: Final = 0x3BD
XK_eng: Final = 0x3BF
XK_Amacron: Final = 0x3C0
XK_Iogonek: Final = 0x3C7
XK_Eabovedot: Final = 0x3CC
XK_Imacron: Final = 0x3CF
XK_Ncedilla: Final = 0x3D1
XK_Omacron: Final = 0x3D2
XK_Kcedilla: Final = 0x3D3
XK_Uogonek: Final = 0x3D9
XK_Utilde: Final = 0x3DD
XK_Umacron: Final = 0x3DE
XK_amacron: Final = 0x3E0
XK_iogonek: Final = 0x3E7
XK_eabovedot: Final = 0x3EC
XK_imacron: Final = 0x3EF
XK_ncedilla: Final = 0x3F1
XK_omacron: Final = 0x3F2
XK_kcedilla: Final = 0x3F3
XK_uogonek: Final = 0x3F9
XK_utilde: Final = 0x3FD
XK_umacron: Final = 0x3FE
