"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Add a dummy package name. Having no package, like
core/lib/core/error_codes.proto, or having tensorflow.error, like
tsl/protobuf/error_codes.proto, results in name collision errors in generated
code for some users that use JS through J2CL.
"""

import google.protobuf.descriptor
from tensorflow.tsl.protobuf.error_codes_pb2 import (
    ABORTED as ABORTED,
    ALREADY_EXISTS as ALREADY_EXISTS,
    CANCELLED as CANCELLED,
    DATA_LOSS as DATA_LOSS,
    DEADLINE_EXCEEDED as DEADLINE_EXCEEDED,
    DO_NOT_USE_RESERVED_FOR_FUTURE_EXPANSION_USE_DEFAULT_IN_SWITCH_INSTEAD_ as DO_NOT_USE_RESERVED_FOR_FUTURE_EXPANSION_USE_DEFAULT_IN_SWITCH_INSTEAD_,
    FAILED_PRECONDITION as FAILED_PRECONDITION,
    INTERNAL as INTERNAL,
    INVALID_ARGUMENT as INVALID_ARGUMENT,
    NOT_FOUND as NOT_FOUND,
    OK as OK,
    OUT_OF_RANGE as OUT_OF_RANGE,
    PERMISSION_DENIED as PERMISSION_DENIED,
    RESOURCE_EXHAUSTED as RESOURCE_EXHAUSTED,
    UNAUTHENTICATED as UNAUTHENTICATED,
    UNAVAILABLE as UNAVAILABLE,
    UNIMPLEMENTED as UNIMPLEMENTED,
    UNKNOWN as UNKNOWN,
    Code as Code,
)

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor
