from typing import Final

class PaymentInstrumentType:
    AmexExpressCheckoutCard: Final = "amex_express_checkout_card"
    AndroidPayCard: Final = "android_pay_card"
    ApplePayCard: Final = "apple_pay_card"
    CreditCard: Final = "credit_card"
    EuropeBankAccount: Final = "europe_bank_account"
    LocalPayment: Final = "local_payment"
    MasterpassCard: Final = "masterpass_card"
    MetaCheckoutCard: Final = "meta_checkout_card"
    MetaCheckoutToken: Final = "meta_checkout_token"
    PayPalAccount: Final = "paypal_account"
    PayPalHere: Final = "paypal_here"
    SamsungPayCard: Final = "samsung_pay_card"
    SepaDirectDebitAccount: Final = "sepa_debit_account"
    UsBankAccount: Final = "us_bank_account"
    VenmoAccount: Final = "venmo_account"
    VisaCheckoutCard: Final = "visa_checkout_card"
