"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import google.protobuf.descriptor
from tensorflow.tsl.protobuf.rpc_options_pb2 import RPCOptions as RPCOptions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor
