from hvac.api.system_backend.system_backend_mixin import SystemBackendMixin

class Seal(SystemBackendMixin):
    def is_sealed(self): ...
    def read_seal_status(self): ...
    def seal(self): ...
    def submit_unseal_key(self, key=None, reset: bool = False, migrate: bool = False): ...
    def submit_unseal_keys(self, keys, migrate: bool = False): ...
