import datetime

def to_datetime(jdc: float) -> datetime.datetime: ...
def from_datetime(dt: datetime.datetime) -> float: ...
def to_gregorian(jdc: float) -> tuple[int, int, int]: ...
def from_gregorian(year: int, month: int, day: int) -> float: ...
def to_julian(jdc: float) -> tuple[int, int, int]: ...
def from_julian(year: int, month: int, day: int) -> float: ...
