from _typeshed import Incomplete
from collections.abc import Callable

def decorate_all_functions(function_decorator: Callable[..., Incomplete]) -> Callable[..., Incomplete]: ...
def xray_on_call(cls, func: Callable[..., Incomplete]) -> Callable[..., Incomplete]: ...
def parse_bind(bind) -> dict[str, str]: ...
