from ..core import WesternCalendar

class Spain(WesternCalendar): ...
class Andalusia(Spain): ...
class Aragon(Spain): ...
class CastileAndLeon(Spain): ...
class CastillaLaMancha(Spain): ...
class CanaryIslands(Spain): ...
class Catalonia(Spain): ...
class Extremadura(Spain): ...
class Galicia(Spain): ...
class BalearicIslands(Spain): ...
class LaRioja(Spain): ...
class CommunityofMadrid(Spain): ...
class Murcia(Spain): ...
class Navarre(Spain): ...
class Asturias(Spain): ...
class BasqueCountry(Spain): ...
class Cantabria(Spain): ...
class ValencianCommunity(Spain): ...
