from typing import Final, Literal

EPOCH: Final = 2374479.5
YEAR_EPOCH: Final = 1789
DAYS_IN_YEAR: Final = 365
MONTHS: Final[tuple[str, ...]]

def legal_date(year: int, month: int, day: int) -> Literal[True]: ...
def to_jd(year: int, month: int, day: int) -> float: ...
def from_jd(jd: float) -> tuple[int, int, int]: ...
def from_gregorian(year: int, month: int, day: int) -> tuple[int, int, int]: ...
def to_gregorian(year: int, month: int, day: int) -> tuple[int, int, int]: ...
def dayname(year: int, month: int, day: int) -> tuple[str, str]: ...
def weekday(day: int) -> int: ...
def festival(month: int, day: int) -> str | None: ...
