class AuthlibDeprecationWarning(DeprecationWarning): ...

def deprecate(
    message: str, version: str | None = None, link_uid: str | None = None, link_file: str | None = None, stacklevel: int = 3
) -> None: ...
