from networkx.utils.backends import _dispatchable

__all__ = ["karate_club_graph", "davis_southern_women_graph", "florentine_families_graph", "les_miserables_graph"]

@_dispatchable
def karate_club_graph(): ...
@_dispatchable
def davis_southern_women_graph(): ...
@_dispatchable
def florentine_families_graph(): ...
@_dispatchable
def les_miserables_graph(): ...
