from ._base import Trie as ABCTrie

class Trie(ABCTrie):
    def __init__(self, data) -> None: ...
    def __contains__(self, key): ...
    def __len__(self) -> int: ...
    def __iter__(self): ...
    def __getitem__(self, key): ...
    def keys(self, prefix=None): ...
    def has_keys_with_prefix(self, prefix): ...
