class ISAPIError(Exception):
    errno: int
    strerror: str | None
    funcname: str | None
    def __init__(self, errno: int, strerror: str | None = None, funcname: str | None = None) -> None: ...

class FilterError(ISAPIError): ...
class ExtensionError(ISAPIError): ...
class InternalReloadException(Exception): ...
