from typing import Final

DAEMON_ADDRESS_KEY: Final = "AWS_XRAY_DAEMON_ADDRESS"
DEFAULT_ADDRESS: Final = "127.0.0.1:2000"

class DaemonConfig:
    def __init__(self, daemon_address: str | None = "127.0.0.1:2000") -> None: ...
    @property
    def udp_ip(self) -> str: ...
    @property
    def udp_port(self) -> int: ...
    @property
    def tcp_ip(self) -> str: ...
    @property
    def tcp_port(self) -> int: ...
