from pymeeus.Angle import Angle
from pymeeus.Epoch import Epoch

class Minor:
    def __init__(self, q: float, e: float, i: Angle, omega: Angle, w: Angle, t: Epoch) -> None: ...
    def set(self, q: float, e: float, i: Angle, omega: Angle, w: Angle, t: Epoch) -> None: ...
    def geocentric_position(self, epoch: Epoch) -> tuple[Angle, Angle, Angle]: ...
    def heliocentric_ecliptical_position(self, epoch: Epoch) -> tuple[Angle, Angle]: ...

def main() -> None: ...
