from typing import Final

XK_blank: Final = 0x9DF
XK_soliddiamond: Final = 0x9E0
XK_checkerboard: Final = 0x9E1
XK_ht: Final = 0x9E2
XK_ff: Final = 0x9E3
XK_cr: Final = 0x9E4
XK_lf: Final = 0x9E5
XK_nl: Final = 0x9E8
XK_vt: Final = 0x9E9
XK_lowrightcorner: Final = 0x9EA
XK_uprightcorner: Final = 0x9EB
XK_upleftcorner: Final = 0x9EC
XK_lowleftcorner: Final = 0x9ED
XK_crossinglines: Final = 0x9EE
XK_horizlinescan1: Final = 0x9EF
XK_horizlinescan3: Final = 0x9F0
XK_horizlinescan5: Final = 0x9F1
XK_horizlinescan7: Final = 0x9F2
XK_horizlinescan9: Final = 0x9F3
XK_leftt: Final = 0x9F4
XK_rightt: Final = 0x9F5
XK_bott: Final = 0x9F6
XK_topt: Final = 0x9F7
XK_vertbar: Final = 0x9F8
