from _typeshed import StrOrBytesPath
from collections.abc import Iterable

def rmtree(path: StrOrBytesPath) -> None: ...
def isdir(path: StrOrBytesPath) -> bool: ...
def listdir(path: StrOrBytesPath) -> list[str]: ...
def exists(path: StrOrBytesPath) -> bool: ...
def copy(src: StrOrBytesPath, dst: StrOrBytesPath, overwrite: bool = False) -> None: ...
def makedirs(path: StrOrBytesPath) -> None: ...
def glob(pattern: str | bytes | Iterable[str | bytes]) -> list[str]: ...
def __getattr__(name: str): ...  # incomplete module
