from .assertpy import (
    WarningLoggingAdapter as WarningLoggingAdapter,
    __version__ as __version__,
    add_extension as add_extension,
    assert_that as assert_that,
    assert_warn as assert_warn,
    fail as fail,
    remove_extension as remove_extension,
    soft_assertions as soft_assertions,
    soft_fail as soft_fail,
)
from .file import contents_of as contents_of
