from aws_xray_sdk.ext.dbapi2 import XRayTracedConn

def patch() -> None: ...

class XRayTracedSQLite(XRayTracedConn):
    def execute(self, *args, **kwargs): ...
    def executemany(self, *args, **kwargs): ...
