from typing import Final

ERROR_FIRST: Final = 1000
HASHCHK: Final = 1000
NISAMCHK: Final = 1001
NO: Final = 1002
YES: Final = 1003
CANT_CREATE_FILE: Final = 1004
CANT_CREATE_TABLE: Final = 1005
CANT_CREATE_DB: Final = 1006
DB_CREATE_EXISTS: Final = 1007
DB_DROP_EXISTS: Final = 1008
DB_DROP_DELETE: Final = 1009
DB_DROP_RMDIR: Final = 1010
CANT_DELETE_FILE: Final = 1011
CANT_FIND_SYSTEM_REC: Final = 1012
CANT_GET_STAT: Final = 1013
CANT_GET_WD: Final = 1014
CANT_LOCK: Final = 1015
CANT_OPEN_FILE: Final = 1016
FILE_NOT_FOUND: Final = 1017
CANT_READ_DIR: Final = 1018
CANT_SET_WD: Final = 1019
CHECKREAD: Final = 1020
DISK_FULL: Final = 1021
DUP_KEY: Final = 1022
ERROR_ON_CLOSE: Final = 1023
ERROR_ON_READ: Final = 1024
ERROR_ON_RENAME: Final = 1025
ERROR_ON_WRITE: Final = 1026
FILE_USED: Final = 1027
FILSORT_ABORT: Final = 1028
FORM_NOT_FOUND: Final = 1029
GET_ERRNO: Final = 1030
ILLEGAL_HA: Final = 1031
KEY_NOT_FOUND: Final = 1032
NOT_FORM_FILE: Final = 1033
NOT_KEYFILE: Final = 1034
OLD_KEYFILE: Final = 1035
OPEN_AS_READONLY: Final = 1036
OUTOFMEMORY: Final = 1037
OUT_OF_SORTMEMORY: Final = 1038
UNEXPECTED_EOF: Final = 1039
CON_COUNT_ERROR: Final = 1040
OUT_OF_RESOURCES: Final = 1041
BAD_HOST_ERROR: Final = 1042
HANDSHAKE_ERROR: Final = 1043
DBACCESS_DENIED_ERROR: Final = 1044
ACCESS_DENIED_ERROR: Final = 1045
NO_DB_ERROR: Final = 1046
UNKNOWN_COM_ERROR: Final = 1047
BAD_NULL_ERROR: Final = 1048
BAD_DB_ERROR: Final = 1049
TABLE_EXISTS_ERROR: Final = 1050
BAD_TABLE_ERROR: Final = 1051
NON_UNIQ_ERROR: Final = 1052
SERVER_SHUTDOWN: Final = 1053
BAD_FIELD_ERROR: Final = 1054
WRONG_FIELD_WITH_GROUP: Final = 1055
WRONG_GROUP_FIELD: Final = 1056
WRONG_SUM_SELECT: Final = 1057
WRONG_VALUE_COUNT: Final = 1058
TOO_LONG_IDENT: Final = 1059
DUP_FIELDNAME: Final = 1060
DUP_KEYNAME: Final = 1061
DUP_ENTRY: Final = 1062
WRONG_FIELD_SPEC: Final = 1063
PARSE_ERROR: Final = 1064
EMPTY_QUERY: Final = 1065
NONUNIQ_TABLE: Final = 1066
INVALID_DEFAULT: Final = 1067
MULTIPLE_PRI_KEY: Final = 1068
TOO_MANY_KEYS: Final = 1069
TOO_MANY_KEY_PARTS: Final = 1070
TOO_LONG_KEY: Final = 1071
KEY_COLUMN_DOES_NOT_EXITS: Final = 1072
BLOB_USED_AS_KEY: Final = 1073
TOO_BIG_FIELDLENGTH: Final = 1074
WRONG_AUTO_KEY: Final = 1075
READY: Final = 1076
NORMAL_SHUTDOWN: Final = 1077
GOT_SIGNAL: Final = 1078
SHUTDOWN_COMPLETE: Final = 1079
FORCING_CLOSE: Final = 1080
IPSOCK_ERROR: Final = 1081
NO_SUCH_INDEX: Final = 1082
WRONG_FIELD_TERMINATORS: Final = 1083
BLOBS_AND_NO_TERMINATED: Final = 1084
TEXTFILE_NOT_READABLE: Final = 1085
FILE_EXISTS_ERROR: Final = 1086
LOAD_INFO: Final = 1087
ALTER_INFO: Final = 1088
WRONG_SUB_KEY: Final = 1089
CANT_REMOVE_ALL_FIELDS: Final = 1090
CANT_DROP_FIELD_OR_KEY: Final = 1091
INSERT_INFO: Final = 1092
UPDATE_TABLE_USED: Final = 1093
NO_SUCH_THREAD: Final = 1094
KILL_DENIED_ERROR: Final = 1095
NO_TABLES_USED: Final = 1096
TOO_BIG_SET: Final = 1097
NO_UNIQUE_LOGFILE: Final = 1098
TABLE_NOT_LOCKED_FOR_WRITE: Final = 1099
TABLE_NOT_LOCKED: Final = 1100
BLOB_CANT_HAVE_DEFAULT: Final = 1101
WRONG_DB_NAME: Final = 1102
WRONG_TABLE_NAME: Final = 1103
TOO_BIG_SELECT: Final = 1104
UNKNOWN_ERROR: Final = 1105
UNKNOWN_PROCEDURE: Final = 1106
WRONG_PARAMCOUNT_TO_PROCEDURE: Final = 1107
WRONG_PARAMETERS_TO_PROCEDURE: Final = 1108
UNKNOWN_TABLE: Final = 1109
FIELD_SPECIFIED_TWICE: Final = 1110
INVALID_GROUP_FUNC_USE: Final = 1111
UNSUPPORTED_EXTENSION: Final = 1112
TABLE_MUST_HAVE_COLUMNS: Final = 1113
RECORD_FILE_FULL: Final = 1114
UNKNOWN_CHARACTER_SET: Final = 1115
TOO_MANY_TABLES: Final = 1116
TOO_MANY_FIELDS: Final = 1117
TOO_BIG_ROWSIZE: Final = 1118
STACK_OVERRUN: Final = 1119
WRONG_OUTER_JOIN: Final = 1120
NULL_COLUMN_IN_INDEX: Final = 1121
CANT_FIND_UDF: Final = 1122
CANT_INITIALIZE_UDF: Final = 1123
UDF_NO_PATHS: Final = 1124
UDF_EXISTS: Final = 1125
CANT_OPEN_LIBRARY: Final = 1126
CANT_FIND_DL_ENTRY: Final = 1127
FUNCTION_NOT_DEFINED: Final = 1128
HOST_IS_BLOCKED: Final = 1129
HOST_NOT_PRIVILEGED: Final = 1130
PASSWORD_ANONYMOUS_USER: Final = 1131
PASSWORD_NOT_ALLOWED: Final = 1132
PASSWORD_NO_MATCH: Final = 1133
UPDATE_INFO: Final = 1134
CANT_CREATE_THREAD: Final = 1135
WRONG_VALUE_COUNT_ON_ROW: Final = 1136
CANT_REOPEN_TABLE: Final = 1137
INVALID_USE_OF_NULL: Final = 1138
REGEXP_ERROR: Final = 1139
MIX_OF_GROUP_FUNC_AND_FIELDS: Final = 1140
NONEXISTING_GRANT: Final = 1141
TABLEACCESS_DENIED_ERROR: Final = 1142
COLUMNACCESS_DENIED_ERROR: Final = 1143
ILLEGAL_GRANT_FOR_TABLE: Final = 1144
GRANT_WRONG_HOST_OR_USER: Final = 1145
NO_SUCH_TABLE: Final = 1146
NONEXISTING_TABLE_GRANT: Final = 1147
NOT_ALLOWED_COMMAND: Final = 1148
SYNTAX_ERROR: Final = 1149
DELAYED_CANT_CHANGE_LOCK: Final = 1150
TOO_MANY_DELAYED_THREADS: Final = 1151
ABORTING_CONNECTION: Final = 1152
NET_PACKET_TOO_LARGE: Final = 1153
NET_READ_ERROR_FROM_PIPE: Final = 1154
NET_FCNTL_ERROR: Final = 1155
NET_PACKETS_OUT_OF_ORDER: Final = 1156
NET_UNCOMPRESS_ERROR: Final = 1157
NET_READ_ERROR: Final = 1158
NET_READ_INTERRUPTED: Final = 1159
NET_ERROR_ON_WRITE: Final = 1160
NET_WRITE_INTERRUPTED: Final = 1161
TOO_LONG_STRING: Final = 1162
TABLE_CANT_HANDLE_BLOB: Final = 1163
TABLE_CANT_HANDLE_AUTO_INCREMENT: Final = 1164
DELAYED_INSERT_TABLE_LOCKED: Final = 1165
WRONG_COLUMN_NAME: Final = 1166
WRONG_KEY_COLUMN: Final = 1167
WRONG_MRG_TABLE: Final = 1168
DUP_UNIQUE: Final = 1169
BLOB_KEY_WITHOUT_LENGTH: Final = 1170
PRIMARY_CANT_HAVE_NULL: Final = 1171
TOO_MANY_ROWS: Final = 1172
REQUIRES_PRIMARY_KEY: Final = 1173
NO_RAID_COMPILED: Final = 1174
UPDATE_WITHOUT_KEY_IN_SAFE_MODE: Final = 1175
KEY_DOES_NOT_EXITS: Final = 1176
CHECK_NO_SUCH_TABLE: Final = 1177
CHECK_NOT_IMPLEMENTED: Final = 1178
CANT_DO_THIS_DURING_AN_TRANSACTION: Final = 1179
ERROR_DURING_COMMIT: Final = 1180
ERROR_DURING_ROLLBACK: Final = 1181
ERROR_DURING_FLUSH_LOGS: Final = 1182
ERROR_DURING_CHECKPOINT: Final = 1183
NEW_ABORTING_CONNECTION: Final = 1184
DUMP_NOT_IMPLEMENTED: Final = 1185
FLUSH_MASTER_BINLOG_CLOSED: Final = 1186
INDEX_REBUILD: Final = 1187
MASTER: Final = 1188
MASTER_NET_READ: Final = 1189
MASTER_NET_WRITE: Final = 1190
FT_MATCHING_KEY_NOT_FOUND: Final = 1191
LOCK_OR_ACTIVE_TRANSACTION: Final = 1192
UNKNOWN_SYSTEM_VARIABLE: Final = 1193
CRASHED_ON_USAGE: Final = 1194
CRASHED_ON_REPAIR: Final = 1195
WARNING_NOT_COMPLETE_ROLLBACK: Final = 1196
TRANS_CACHE_FULL: Final = 1197
SLAVE_MUST_STOP: Final = 1198
SLAVE_NOT_RUNNING: Final = 1199
BAD_SLAVE: Final = 1200
MASTER_INFO: Final = 1201
SLAVE_THREAD: Final = 1202
TOO_MANY_USER_CONNECTIONS: Final = 1203
SET_CONSTANTS_ONLY: Final = 1204
LOCK_WAIT_TIMEOUT: Final = 1205
LOCK_TABLE_FULL: Final = 1206
READ_ONLY_TRANSACTION: Final = 1207
DROP_DB_WITH_READ_LOCK: Final = 1208
CREATE_DB_WITH_READ_LOCK: Final = 1209
WRONG_ARGUMENTS: Final = 1210
NO_PERMISSION_TO_CREATE_USER: Final = 1211
UNION_TABLES_IN_DIFFERENT_DIR: Final = 1212
LOCK_DEADLOCK: Final = 1213
TABLE_CANT_HANDLE_FT: Final = 1214
CANNOT_ADD_FOREIGN: Final = 1215
NO_REFERENCED_ROW: Final = 1216
ROW_IS_REFERENCED: Final = 1217
CONNECT_TO_MASTER: Final = 1218
QUERY_ON_MASTER: Final = 1219
ERROR_WHEN_EXECUTING_COMMAND: Final = 1220
WRONG_USAGE: Final = 1221
WRONG_NUMBER_OF_COLUMNS_IN_SELECT: Final = 1222
CANT_UPDATE_WITH_READLOCK: Final = 1223
MIXING_NOT_ALLOWED: Final = 1224
DUP_ARGUMENT: Final = 1225
USER_LIMIT_REACHED: Final = 1226
SPECIFIC_ACCESS_DENIED_ERROR: Final = 1227
LOCAL_VARIABLE: Final = 1228
GLOBAL_VARIABLE: Final = 1229
NO_DEFAULT: Final = 1230
WRONG_VALUE_FOR_VAR: Final = 1231
WRONG_TYPE_FOR_VAR: Final = 1232
VAR_CANT_BE_READ: Final = 1233
CANT_USE_OPTION_HERE: Final = 1234
NOT_SUPPORTED_YET: Final = 1235
MASTER_FATAL_ERROR_READING_BINLOG: Final = 1236
SLAVE_IGNORED_TABLE: Final = 1237
INCORRECT_GLOBAL_LOCAL_VAR: Final = 1238
WRONG_FK_DEF: Final = 1239
KEY_REF_DO_NOT_MATCH_TABLE_REF: Final = 1240
OPERAND_COLUMNS: Final = 1241
SUBQUERY_NO_1_ROW: Final = 1242
UNKNOWN_STMT_HANDLER: Final = 1243
CORRUPT_HELP_DB: Final = 1244
CYCLIC_REFERENCE: Final = 1245
AUTO_CONVERT: Final = 1246
ILLEGAL_REFERENCE: Final = 1247
DERIVED_MUST_HAVE_ALIAS: Final = 1248
SELECT_REDUCED: Final = 1249
TABLENAME_NOT_ALLOWED_HERE: Final = 1250
NOT_SUPPORTED_AUTH_MODE: Final = 1251
SPATIAL_CANT_HAVE_NULL: Final = 1252
COLLATION_CHARSET_MISMATCH: Final = 1253
SLAVE_WAS_RUNNING: Final = 1254
SLAVE_WAS_NOT_RUNNING: Final = 1255
TOO_BIG_FOR_UNCOMPRESS: Final = 1256
ZLIB_Z_MEM_ERROR: Final = 1257
ZLIB_Z_BUF_ERROR: Final = 1258
ZLIB_Z_DATA_ERROR: Final = 1259
CUT_VALUE_GROUP_CONCAT: Final = 1260
WARN_TOO_FEW_RECORDS: Final = 1261
WARN_TOO_MANY_RECORDS: Final = 1262
WARN_NULL_TO_NOTNULL: Final = 1263
WARN_DATA_OUT_OF_RANGE: Final = 1264
WARN_DATA_TRUNCATED: Final = 1265
WARN_USING_OTHER_HANDLER: Final = 1266
CANT_AGGREGATE_2COLLATIONS: Final = 1267
DROP_USER: Final = 1268
REVOKE_GRANTS: Final = 1269
CANT_AGGREGATE_3COLLATIONS: Final = 1270
CANT_AGGREGATE_NCOLLATIONS: Final = 1271
VARIABLE_IS_NOT_STRUCT: Final = 1272
UNKNOWN_COLLATION: Final = 1273
SLAVE_IGNORED_SSL_PARAMS: Final = 1274
SERVER_IS_IN_SECURE_AUTH_MODE: Final = 1275
WARN_FIELD_RESOLVED: Final = 1276
BAD_SLAVE_UNTIL_COND: Final = 1277
MISSING_SKIP_SLAVE: Final = 1278
UNTIL_COND_IGNORED: Final = 1279
WRONG_NAME_FOR_INDEX: Final = 1280
WRONG_NAME_FOR_CATALOG: Final = 1281
WARN_QC_RESIZE: Final = 1282
BAD_FT_COLUMN: Final = 1283
UNKNOWN_KEY_CACHE: Final = 1284
WARN_HOSTNAME_WONT_WORK: Final = 1285
UNKNOWN_STORAGE_ENGINE: Final = 1286
WARN_DEPRECATED_SYNTAX: Final = 1287
NON_UPDATABLE_TABLE: Final = 1288
FEATURE_DISABLED: Final = 1289
OPTION_PREVENTS_STATEMENT: Final = 1290
DUPLICATED_VALUE_IN_TYPE: Final = 1291
TRUNCATED_WRONG_VALUE: Final = 1292
TOO_MUCH_AUTO_TIMESTAMP_COLS: Final = 1293
INVALID_ON_UPDATE: Final = 1294
UNSUPPORTED_PS: Final = 1295
GET_ERRMSG: Final = 1296
GET_TEMPORARY_ERRMSG: Final = 1297
UNKNOWN_TIME_ZONE: Final = 1298
WARN_INVALID_TIMESTAMP: Final = 1299
INVALID_CHARACTER_STRING: Final = 1300
WARN_ALLOWED_PACKET_OVERFLOWED: Final = 1301
CONFLICTING_DECLARATIONS: Final = 1302
SP_NO_RECURSIVE_CREATE: Final = 1303
SP_ALREADY_EXISTS: Final = 1304
SP_DOES_NOT_EXIST: Final = 1305
SP_DROP_FAILED: Final = 1306
SP_STORE_FAILED: Final = 1307
SP_LILABEL_MISMATCH: Final = 1308
SP_LABEL_REDEFINE: Final = 1309
SP_LABEL_MISMATCH: Final = 1310
SP_UNINIT_VAR: Final = 1311
SP_BADSELECT: Final = 1312
SP_BADRETURN: Final = 1313
SP_BADSTATEMENT: Final = 1314
UPDATE_LOG_DEPRECATED_IGNORED: Final = 1315
UPDATE_LOG_DEPRECATED_TRANSLATED: Final = 1316
QUERY_INTERRUPTED: Final = 1317
SP_WRONG_NO_OF_ARGS: Final = 1318
SP_COND_MISMATCH: Final = 1319
SP_NORETURN: Final = 1320
SP_NORETURNEND: Final = 1321
SP_BAD_CURSOR_QUERY: Final = 1322
SP_BAD_CURSOR_SELECT: Final = 1323
SP_CURSOR_MISMATCH: Final = 1324
SP_CURSOR_ALREADY_OPEN: Final = 1325
SP_CURSOR_NOT_OPEN: Final = 1326
SP_UNDECLARED_VAR: Final = 1327
SP_WRONG_NO_OF_FETCH_ARGS: Final = 1328
SP_FETCH_NO_DATA: Final = 1329
SP_DUP_PARAM: Final = 1330
SP_DUP_VAR: Final = 1331
SP_DUP_COND: Final = 1332
SP_DUP_CURS: Final = 1333
SP_CANT_ALTER: Final = 1334
SP_SUBSELECT_NYI: Final = 1335
STMT_NOT_ALLOWED_IN_SF_OR_TRG: Final = 1336
SP_VARCOND_AFTER_CURSHNDLR: Final = 1337
SP_CURSOR_AFTER_HANDLER: Final = 1338
SP_CASE_NOT_FOUND: Final = 1339
FPARSER_TOO_BIG_FILE: Final = 1340
FPARSER_BAD_HEADER: Final = 1341
FPARSER_EOF_IN_COMMENT: Final = 1342
FPARSER_ERROR_IN_PARAMETER: Final = 1343
FPARSER_EOF_IN_UNKNOWN_PARAMETER: Final = 1344
VIEW_NO_EXPLAIN: Final = 1345
FRM_UNKNOWN_TYPE: Final = 1346
WRONG_OBJECT: Final = 1347
NONUPDATEABLE_COLUMN: Final = 1348
VIEW_SELECT_DERIVED: Final = 1349
VIEW_SELECT_CLAUSE: Final = 1350
VIEW_SELECT_VARIABLE: Final = 1351
VIEW_SELECT_TMPTABLE: Final = 1352
VIEW_WRONG_LIST: Final = 1353
WARN_VIEW_MERGE: Final = 1354
WARN_VIEW_WITHOUT_KEY: Final = 1355
VIEW_INVALID: Final = 1356
SP_NO_DROP_SP: Final = 1357
SP_GOTO_IN_HNDLR: Final = 1358
TRG_ALREADY_EXISTS: Final = 1359
TRG_DOES_NOT_EXIST: Final = 1360
TRG_ON_VIEW_OR_TEMP_TABLE: Final = 1361
TRG_CANT_CHANGE_ROW: Final = 1362
TRG_NO_SUCH_ROW_IN_TRG: Final = 1363
NO_DEFAULT_FOR_FIELD: Final = 1364
DIVISION_BY_ZERO: Final = 1365
TRUNCATED_WRONG_VALUE_FOR_FIELD: Final = 1366
ILLEGAL_VALUE_FOR_TYPE: Final = 1367
VIEW_NONUPD_CHECK: Final = 1368
VIEW_CHECK_FAILED: Final = 1369
PROCACCESS_DENIED_ERROR: Final = 1370
RELAY_LOG_FAIL: Final = 1371
PASSWD_LENGTH: Final = 1372
UNKNOWN_TARGET_BINLOG: Final = 1373
IO_ERR_LOG_INDEX_READ: Final = 1374
BINLOG_PURGE_PROHIBITED: Final = 1375
FSEEK_FAIL: Final = 1376
BINLOG_PURGE_FATAL_ERR: Final = 1377
LOG_IN_USE: Final = 1378
LOG_PURGE_UNKNOWN_ERR: Final = 1379
RELAY_LOG_INIT: Final = 1380
NO_BINARY_LOGGING: Final = 1381
RESERVED_SYNTAX: Final = 1382
WSAS_FAILED: Final = 1383
DIFF_GROUPS_PROC: Final = 1384
NO_GROUP_FOR_PROC: Final = 1385
ORDER_WITH_PROC: Final = 1386
LOGGING_PROHIBIT_CHANGING_OF: Final = 1387
NO_FILE_MAPPING: Final = 1388
WRONG_MAGIC: Final = 1389
PS_MANY_PARAM: Final = 1390
KEY_PART_0: Final = 1391
VIEW_CHECKSUM: Final = 1392
VIEW_MULTIUPDATE: Final = 1393
VIEW_NO_INSERT_FIELD_LIST: Final = 1394
VIEW_DELETE_MERGE_VIEW: Final = 1395
CANNOT_USER: Final = 1396
XAER_NOTA: Final = 1397
XAER_INVAL: Final = 1398
XAER_RMFAIL: Final = 1399
XAER_OUTSIDE: Final = 1400
XAER_RMERR: Final = 1401
XA_RBROLLBACK: Final = 1402
NONEXISTING_PROC_GRANT: Final = 1403
PROC_AUTO_GRANT_FAIL: Final = 1404
PROC_AUTO_REVOKE_FAIL: Final = 1405
DATA_TOO_LONG: Final = 1406
SP_BAD_SQLSTATE: Final = 1407
STARTUP: Final = 1408
LOAD_FROM_FIXED_SIZE_ROWS_TO_VAR: Final = 1409
CANT_CREATE_USER_WITH_GRANT: Final = 1410
WRONG_VALUE_FOR_TYPE: Final = 1411
TABLE_DEF_CHANGED: Final = 1412
SP_DUP_HANDLER: Final = 1413
SP_NOT_VAR_ARG: Final = 1414
SP_NO_RETSET: Final = 1415
CANT_CREATE_GEOMETRY_OBJECT: Final = 1416
FAILED_ROUTINE_BREAK_BINLOG: Final = 1417
BINLOG_UNSAFE_ROUTINE: Final = 1418
BINLOG_CREATE_ROUTINE_NEED_SUPER: Final = 1419
EXEC_STMT_WITH_OPEN_CURSOR: Final = 1420
STMT_HAS_NO_OPEN_CURSOR: Final = 1421
COMMIT_NOT_ALLOWED_IN_SF_OR_TRG: Final = 1422
NO_DEFAULT_FOR_VIEW_FIELD: Final = 1423
SP_NO_RECURSION: Final = 1424
TOO_BIG_SCALE: Final = 1425
TOO_BIG_PRECISION: Final = 1426
M_BIGGER_THAN_D: Final = 1427
WRONG_LOCK_OF_SYSTEM_TABLE: Final = 1428
CONNECT_TO_FOREIGN_DATA_SOURCE: Final = 1429
QUERY_ON_FOREIGN_DATA_SOURCE: Final = 1430
FOREIGN_DATA_SOURCE_DOESNT_EXIST: Final = 1431
FOREIGN_DATA_STRING_INVALID_CANT_CREATE: Final = 1432
FOREIGN_DATA_STRING_INVALID: Final = 1433
CANT_CREATE_FEDERATED_TABLE: Final = 1434
TRG_IN_WRONG_SCHEMA: Final = 1435
STACK_OVERRUN_NEED_MORE: Final = 1436
TOO_LONG_BODY: Final = 1437
WARN_CANT_DROP_DEFAULT_KEYCACHE: Final = 1438
TOO_BIG_DISPLAYWIDTH: Final = 1439
XAER_DUPID: Final = 1440
DATETIME_FUNCTION_OVERFLOW: Final = 1441
CANT_UPDATE_USED_TABLE_IN_SF_OR_TRG: Final = 1442
VIEW_PREVENT_UPDATE: Final = 1443
PS_NO_RECURSION: Final = 1444
SP_CANT_SET_AUTOCOMMIT: Final = 1445
MALFORMED_DEFINER: Final = 1446
VIEW_FRM_NO_USER: Final = 1447
VIEW_OTHER_USER: Final = 1448
NO_SUCH_USER: Final = 1449
FORBID_SCHEMA_CHANGE: Final = 1450
ROW_IS_REFERENCED_2: Final = 1451
NO_REFERENCED_ROW_2: Final = 1452
SP_BAD_VAR_SHADOW: Final = 1453
TRG_NO_DEFINER: Final = 1454
OLD_FILE_FORMAT: Final = 1455
SP_RECURSION_LIMIT: Final = 1456
SP_PROC_TABLE_CORRUPT: Final = 1457
SP_WRONG_NAME: Final = 1458
TABLE_NEEDS_UPGRADE: Final = 1459
SP_NO_AGGREGATE: Final = 1460
MAX_PREPARED_STMT_COUNT_REACHED: Final = 1461
VIEW_RECURSIVE: Final = 1462
NON_GROUPING_FIELD_USED: Final = 1463
TABLE_CANT_HANDLE_SPKEYS: Final = 1464
NO_TRIGGERS_ON_SYSTEM_SCHEMA: Final = 1465
USERNAME: Final = 1466
HOSTNAME: Final = 1467
WRONG_STRING_LENGTH: Final = 1468
ERROR_LAST: Final = 1468
STATEMENT_TIMEOUT: Final = 1969
QUERY_TIMEOUT: Final = 3024
CONSTRAINT_FAILED: Final = 4025
