############################################################################
# Copyright(c) Open Law Library. All rights reserved.                      #
# See ThirdPartyNotices.txt in the project root for additional notices.    #
#                                                                          #
# Licensed under the Apache License, Version 2.0 (the "License")           #
# you may not use this file except in compliance with the License.         #
# You may obtain a copy of the License at                                  #
#                                                                          #
#     http: // www.apache.org/licenses/LICENSE-2.0                         #
#                                                                          #
# Unless required by applicable law or agreed to in writing, software      #
# distributed under the License is distributed on an "AS IS" BASIS,        #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. #
# See the License for the specific language governing permissions and      #
# limitations under the License.                                           #
############################################################################

# Dynamically assigned attributes
ATTR_EXECUTE_IN_THREAD = "execute_in_thread"
ATTR_COMMAND_TYPE = "command"
ATTR_FEATURE_TYPE = "feature"
ATTR_REGISTERED_NAME = "reg_name"
ATTR_REGISTERED_TYPE = "reg_type"

# Parameters
PARAM_LS = "ls"
