from typing import Final

SERVER_STATUS_IN_TRANS: Final = 1
SERVER_STATUS_AUTOCOMMIT: Final = 2
SERVER_MORE_RESULTS_EXISTS: Final = 8
SERVER_QUERY_NO_GOOD_INDEX_USED: Final = 16
SERVER_QUERY_NO_INDEX_USED: Final = 32
SERVER_STATUS_CURSOR_EXISTS: Final = 64
SERVER_STATUS_LAST_ROW_SENT: Final = 128
SERVER_STATUS_DB_DROPPED: Final = 256
SERVER_STATUS_NO_BACKSLASH_ESCAPES: Final = 512
SERVER_STATUS_METADATA_CHANGED: Final = 1024
