def patch() -> None: ...

class SyncInstrumentedTransport:
    def __init__(self, transport): ...
    def handle_request(self, request): ...

class AsyncInstrumentedTransport:
    def __init__(self, transport): ...
    async def handle_async_request(self, request): ...
