import datetime
from _typeshed import Incomplete
from math import _SupportsFloatOrIndex
from typing import Final

hour: Final[float]
minute: Final[float]
second: Final[float]
newton_precision: Final[float]

def calculate_equinoxes(year, timezone: str = "UTC") -> tuple[Incomplete, Incomplete]: ...
def get_current_longitude(current_date, earth, sun): ...
def newton(f, x0, x1, precision=..., **func_kwargs): ...
def newton_angle_function(t, ts, target_angle, body1, body2): ...
def solar_term(year: int, degrees: _SupportsFloatOrIndex, timezone: str = "UTC") -> datetime.date: ...
