def prepare_grant_uri(uri, client_id, response_type, redirect_uri=None, scope=None, state=None, **kwargs): ...
def prepare_token_request(grant_type, body: str = "", redirect_uri=None, **kwargs): ...
def parse_authorization_code_response(uri, state=None): ...
def parse_implicit_response(uri, state=None): ...
