from typing import Final

mathbb: Final[dict[str, str]]
mathbf: Final[dict[str, str]]
mathbfit: Final[dict[str, str]]
mathcal: Final[dict[str, str]]
mathfrak: Final[dict[str, str]]
mathit: Final[dict[str, str]]
mathsf: Final[dict[str, str]]
mathsfbf: Final[dict[str, str]]
mathsfbfit: Final[dict[str, str]]
mathsfit: Final[dict[str, str]]
mathtt: Final[dict[str, str]]
