import datetime

from ..core import WesternCalendar

class Paraguay(WesternCalendar):
    def get_heroes_day(self, year: int) -> tuple[datetime.date, str]: ...
    def get_founding_of_asuncion(self, year: int) -> tuple[datetime.date, str]: ...
    def get_boqueron_battle_victory_day(self, year: int) -> tuple[datetime.date, str]: ...
