from Xlib.ext import (
    composite as composite,
    damage as damage,
    dpms as dpms,
    ge as ge,
    nvcontrol as nvcontrol,
    randr as randr,
    record as record,
    res as res,
    screensaver as screensaver,
    security as security,
    shape as shape,
    xfixes as xfixes,
    xinerama as xinerama,
    xinput as xinput,
    xtest as xtest,
)

__all__ = [
    "ge",
    "xtest",
    "shape",
    "xinerama",
    "record",
    "composite",
    "randr",
    "xfixes",
    "security",
    "xinput",
    "nvcontrol",
    "damage",
    "dpms",
    "res",
    "screensaver",
]
