from typing import Final

XK_Serbian_dje: Final = 0x6A1
XK_Macedonia_gje: Final = 0x6A2
XK_Cyrillic_io: Final = 0x6A3
XK_Ukrainian_ie: Final = 0x6A4
XK_Ukranian_je: Final = 0x6A4
XK_Macedonia_dse: Final = 0x6A5
XK_Ukrainian_i: Final = 0x6A6
XK_Ukranian_i: Final = 0x6A6
XK_Ukrainian_yi: Final = 0x6A7
XK_Ukranian_yi: Final = 0x6A7
XK_Cyrillic_je: Final = 0x6A8
XK_Serbian_je: Final = 0x6A8
XK_Cyrillic_lje: Final = 0x6A9
XK_Serbian_lje: Final = 0x6A9
XK_Cyrillic_nje: Final = 0x6AA
XK_Serbian_nje: Final = 0x6AA
XK_Serbian_tshe: Final = 0x6AB
XK_Macedonia_kje: Final = 0x6AC
XK_Byelorussian_shortu: Final = 0x6AE
XK_Cyrillic_dzhe: Final = 0x6AF
XK_Serbian_dze: Final = 0x6AF
XK_numerosign: Final = 0x6B0
XK_Serbian_DJE: Final = 0x6B1
XK_Macedonia_GJE: Final = 0x6B2
XK_Cyrillic_IO: Final = 0x6B3
XK_Ukrainian_IE: Final = 0x6B4
XK_Ukranian_JE: Final = 0x6B4
XK_Macedonia_DSE: Final = 0x6B5
XK_Ukrainian_I: Final = 0x6B6
XK_Ukranian_I: Final = 0x6B6
XK_Ukrainian_YI: Final = 0x6B7
XK_Ukranian_YI: Final = 0x6B7
XK_Cyrillic_JE: Final = 0x6B8
XK_Serbian_JE: Final = 0x6B8
XK_Cyrillic_LJE: Final = 0x6B9
XK_Serbian_LJE: Final = 0x6B9
XK_Cyrillic_NJE: Final = 0x6BA
XK_Serbian_NJE: Final = 0x6BA
XK_Serbian_TSHE: Final = 0x6BB
XK_Macedonia_KJE: Final = 0x6BC
XK_Byelorussian_SHORTU: Final = 0x6BE
XK_Cyrillic_DZHE: Final = 0x6BF
XK_Serbian_DZE: Final = 0x6BF
XK_Cyrillic_yu: Final = 0x6C0
XK_Cyrillic_a: Final = 0x6C1
XK_Cyrillic_be: Final = 0x6C2
XK_Cyrillic_tse: Final = 0x6C3
XK_Cyrillic_de: Final = 0x6C4
XK_Cyrillic_ie: Final = 0x6C5
XK_Cyrillic_ef: Final = 0x6C6
XK_Cyrillic_ghe: Final = 0x6C7
XK_Cyrillic_ha: Final = 0x6C8
XK_Cyrillic_i: Final = 0x6C9
XK_Cyrillic_shorti: Final = 0x6CA
XK_Cyrillic_ka: Final = 0x6CB
XK_Cyrillic_el: Final = 0x6CC
XK_Cyrillic_em: Final = 0x6CD
XK_Cyrillic_en: Final = 0x6CE
XK_Cyrillic_o: Final = 0x6CF
XK_Cyrillic_pe: Final = 0x6D0
XK_Cyrillic_ya: Final = 0x6D1
XK_Cyrillic_er: Final = 0x6D2
XK_Cyrillic_es: Final = 0x6D3
XK_Cyrillic_te: Final = 0x6D4
XK_Cyrillic_u: Final = 0x6D5
XK_Cyrillic_zhe: Final = 0x6D6
XK_Cyrillic_ve: Final = 0x6D7
XK_Cyrillic_softsign: Final = 0x6D8
XK_Cyrillic_yeru: Final = 0x6D9
XK_Cyrillic_ze: Final = 0x6DA
XK_Cyrillic_sha: Final = 0x6DB
XK_Cyrillic_e: Final = 0x6DC
XK_Cyrillic_shcha: Final = 0x6DD
XK_Cyrillic_che: Final = 0x6DE
XK_Cyrillic_hardsign: Final = 0x6DF
XK_Cyrillic_YU: Final = 0x6E0
XK_Cyrillic_A: Final = 0x6E1
XK_Cyrillic_BE: Final = 0x6E2
XK_Cyrillic_TSE: Final = 0x6E3
XK_Cyrillic_DE: Final = 0x6E4
XK_Cyrillic_IE: Final = 0x6E5
XK_Cyrillic_EF: Final = 0x6E6
XK_Cyrillic_GHE: Final = 0x6E7
XK_Cyrillic_HA: Final = 0x6E8
XK_Cyrillic_I: Final = 0x6E9
XK_Cyrillic_SHORTI: Final = 0x6EA
XK_Cyrillic_KA: Final = 0x6EB
XK_Cyrillic_EL: Final = 0x6EC
XK_Cyrillic_EM: Final = 0x6ED
XK_Cyrillic_EN: Final = 0x6EE
XK_Cyrillic_O: Final = 0x6EF
XK_Cyrillic_PE: Final = 0x6F0
XK_Cyrillic_YA: Final = 0x6F1
XK_Cyrillic_ER: Final = 0x6F2
XK_Cyrillic_ES: Final = 0x6F3
XK_Cyrillic_TE: Final = 0x6F4
XK_Cyrillic_U: Final = 0x6F5
XK_Cyrillic_ZHE: Final = 0x6F6
XK_Cyrillic_VE: Final = 0x6F7
XK_Cyrillic_SOFTSIGN: Final = 0x6F8
XK_Cyrillic_YERU: Final = 0x6F9
XK_Cyrillic_ZE: Final = 0x6FA
XK_Cyrillic_SHA: Final = 0x6FB
XK_Cyrillic_E: Final = 0x6FC
XK_Cyrillic_SHCHA: Final = 0x6FD
XK_Cyrillic_CHE: Final = 0x6FE
XK_Cyrillic_HARDSIGN: Final = 0x6FF
