def sign_jwt_bearer_assertion(
    key, issuer, audience, subject=None, issued_at=None, expires_at=None, claims=None, header=None, **kwargs
): ...
def client_secret_jwt_sign(client_secret, client_id, token_endpoint, alg: str = "HS256", claims=None, **kwargs): ...
def private_key_jwt_sign(private_key, client_id, token_endpoint, alg: str = "RS256", claims=None, **kwargs): ...
