from .npipeconn import NpipeHTTPAdapter as NpipeHTTPAdapter
from .npipesocket import NpipeSocket as NpipeSocket
from .sshconn import SSHHTTPAdapter as SSHHTTPAdapter
from .unixconn import UnixHTTPAdapter as UnixHTTPAdapter
