from typing import Final, Literal

EPOCH: Final = 2375839.5
YEAR_EPOCH: Final = 1791.0
DAYS_IN_YEAR: Final = 365.0
MOIS: Final[list[str]]
MONTHS: Final[list[str]]
LEAP_CYCLE_DAYS: Final = 1461.0
LEAP_CYCLE_YEARS: Final = 4.0

def leap(year: int, method: Literal[4, 100, 128, "continuous", "madler", "romme", "equinox"] | None = None) -> bool: ...
def premier_da_la_annee(jd: float) -> float: ...
def to_jd(
    year: int, month: int, day: int, method: Literal[4, 100, 128, "continuous", "madler", "romme", "equinox"] | None = None
) -> float: ...
def from_jd(
    jd: float, method: Literal[4, 100, 128, "continuous", "madler", "romme", "equinox"] | None = None
) -> tuple[int, int, int]: ...
def decade(jour: float) -> int: ...
def day_name(month: int, day: int) -> str: ...
def from_gregorian(
    year: int, month: int, day: int, method: Literal[4, 100, 128, "continuous", "madler", "romme", "equinox"] | None = None
) -> tuple[int, int, int]: ...
def to_gregorian(
    an: int, mois: int, jour: int, method: Literal[4, 100, 128, "continuous", "madler", "romme", "equinox"] | None = None
) -> tuple[int, int, int]: ...
def format(an: int, mois: int, jour: int) -> str: ...
