class AtomicCounter:
    value: int
    def __init__(self, initial: int = 0) -> None: ...
    def increment(self, num: int = 1) -> int: ...
    def decrement(self, num: int = 1) -> int: ...
    def get_current(self) -> int: ...
    def reset(self) -> int: ...
