import datetime
from typing import Final

from ..core import WesternCalendar

QUEENS_BIRTHDAY_EXCEPTIONS: Final[dict[int, datetime.date]]

class CaymanIslands(WesternCalendar):
    def get_national_heroes_day(self, year: int) -> datetime.date: ...
    def get_discovery_day(self, year: int) -> datetime.date: ...
    def get_queens_birthday(self, year: int) -> datetime.date: ...
    def get_constitution_day(self, year: int) -> datetime.date: ...
    def get_remembrance_day(self, year: int) -> datetime.date: ...
