class InvalidCalendar(ValueError): ...
class IncompleteComponent(ValueError): ...
class IncompleteAlarmInformation(ValueError): ...
class LocalTimezoneMissing(IncompleteAlarmInformation): ...
class ComponentEndMissing(IncompleteAlarmInformation): ...
class ComponentStartMissing(IncompleteAlarmInformation): ...
class FeatureWillBeRemovedInFutureVersion(DeprecationWarning): ...
class WillBeRemovedInVersion7(FeatureWillBeRemovedInFutureVersion): ...

__all__ = [
    "InvalidCalendar",
    "IncompleteComponent",
    "IncompleteAlarmInformation",
    "LocalTimezoneMissing",
    "ComponentEndMissing",
    "ComponentStartMissing",
    "FeatureWillBeRemovedInFutureVersion",
    "WillBeRemovedInVersion7",
]
