from typing import Final, Literal

EPOCH: Final = 1825029.5
MONTHS: Final[list[str]]
WEEKDAYS: Final = ["Tkyriaka", "Pesnau", "Pshoment", "Peftoou", "Ptiou", "Psoou", "Psabbaton"]

def is_leap(year: int) -> bool: ...
def to_jd(year: int, month: int, day: int) -> float: ...
def from_jd(jdc: float) -> tuple[int, int, int]: ...
def to_gregorian(year: int, month: int, day: int) -> tuple[int, int, int]: ...
def from_gregorian(year: int, month: int, day: int) -> tuple[int, int, int]: ...
def month_length(year: int, month: int) -> Literal[5, 6, 30]: ...
def monthcalendar(year: int, month: int) -> list[list[int | None]]: ...
def format(year: int, month: int, day: int) -> str: ...
