from argparse import ArgumentParser, Namespace

from gunicorn.app.base import Application

from .._types import _WSGIAppType

class WSGIApplication(Application):
    app_uri: str | None

    def init(self, parser: ArgumentParser, opts: Namespace, args: list[str]) -> None: ...
    def load_config(self) -> None: ...
    def load_wsgiapp(self) -> _WSGIAppType: ...
    def load_pasteapp(self) -> _WSGIAppType: ...
    def load(self) -> _WSGIAppType: ...

def run(prog: str | None = None) -> None: ...
