from m3u8.version_matching_rules import VersionMatchingError

def get_version(file_lines: list[str]) -> float | None: ...
def valid_in_all_rules(line_number: int, line: str, version: float) -> list[VersionMatchingError]: ...
def validate(file_lines: list[str]) -> list[VersionMatchingError]: ...
