import datetime

from ..core import WesternCalendar

class Colombia(WesternCalendar):
    def get_epiphany(self, year: int) -> datetime.date: ...
    def get_saint_joseph(self, year: int) -> datetime.date: ...
    def get_ascension(self, year: int) -> datetime.date: ...
    def get_sacred_heart(self, year: int) -> datetime.date: ...
    def get_saint_peter_and_saint_paul(self, year: int) -> datetime.date: ...
    def get_assumption(self, year: int) -> datetime.date: ...
    def get_day_of_the_races(self, year: int) -> datetime.date: ...
    def get_all_saints(self, year: int) -> datetime.date: ...
    def get_cartagena_independence(self, year: int) -> datetime.date: ...
