import datetime

from .core import UnitedStates

class Alabama(UnitedStates): ...
class AlabamaBaldwinCounty(Alabama): ...
class AlabamaMobileCounty(Alabama): ...

class AlabamaPerryCounty(Alabama):
    def get_obama_day(self, year: int) -> tuple[datetime.date, str]: ...
