class ConfigApiMixin:
    def create_config(self, name, data, labels=None, templating=None): ...
    def inspect_config(self, id): ...
    def remove_config(self, id): ...
    def configs(self, filters=None): ...
