from braintree.graphql.enums.recommendations import Recommendations as Recommendations
from braintree.graphql.enums.recommended_payment_option import RecommendedPaymentOption as RecommendedPaymentOption
