"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Copyright 2016 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
==============================================================================
"""

import builtins
import collections.abc
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class MemmappedFileSystemDirectoryElement(google.protobuf.message.Message):
    """A message that describes one region of memmapped file."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    OFFSET_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    LENGTH_FIELD_NUMBER: builtins.int
    offset: builtins.int
    name: builtins.str
    length: builtins.int
    def __init__(
        self, *, offset: builtins.int | None = ..., name: builtins.str | None = ..., length: builtins.int | None = ...
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["length", b"length", "name", b"name", "offset", b"offset"]) -> None: ...

global___MemmappedFileSystemDirectoryElement = MemmappedFileSystemDirectoryElement

@typing.final
class MemmappedFileSystemDirectory(google.protobuf.message.Message):
    """A directory of regions in a memmapped file."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ELEMENT_FIELD_NUMBER: builtins.int
    @property
    def element(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___MemmappedFileSystemDirectoryElement]: ...
    def __init__(
        self, *, element: collections.abc.Iterable[global___MemmappedFileSystemDirectoryElement] | None = ...
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["element", b"element"]) -> None: ...

global___MemmappedFileSystemDirectory = MemmappedFileSystemDirectory
