from typing import Final

def generate_test_payment_method_code(number): ...

VisaPaymentMethodCode: Final = "stub-4111111111111111"
InvalidPaymentMethodCode: Final = "stub-invalid-payment-method-code"

Session: Final = "stub-session"
InvalidSession: Final = "stub-invalid-session"
