class SimpleExtension:
    def GetExtensionVersion(self, vi) -> None: ...
    def HttpExtensionProc(self, control_block) -> int | None: ...
    def TerminateExtension(self, status) -> None: ...

class SimpleFilter:
    filter_flags: int | None
    def GetFilterVersion(self, fv) -> None: ...
    def HttpFilterProc(self, fc) -> None: ...
    def TerminateFilter(self, status) -> None: ...
