"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Protocol messages for describing the results of benchmarks and unit tests."""

import google.protobuf.descriptor
from tensorflow.compiler.xla.tsl.protobuf.test_log_pb2 import (
    AvailableDeviceInfo as AvailableDeviceInfo,
    BenchmarkEntries as BenchmarkEntries,
    BenchmarkEntry as BenchmarkEntry,
    BuildConfiguration as BuildConfiguration,
    CommitId as CommitId,
    CPUInfo as CPUInfo,
    EntryValue as EntryValue,
    GPUInfo as GPUInfo,
    MachineConfiguration as MachineConfiguration,
    MemoryInfo as MemoryInfo,
    MetricEntry as MetricEntry,
    PlatformInfo as PlatformInfo,
    RunConfiguration as RunConfiguration,
    TestResults as TestResults,
)

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor
