from Xlib import (
    XK as XK,
    X as X,
    Xatom as Xatom,
    Xcursorfont as Xcursorfont,
    Xutil as Xutil,
    display as display,
    error as error,
    rdb as rdb,
)

__all__ = ["X", "XK", "Xatom", "Xcursorfont", "Xutil", "display", "error", "rdb"]

# Shared types throughout the stub
