from typing import Literal

from antlr4.Token import Token as Token

class InputStream:
    __slots__ = ("name", "strdata", "_index", "data", "_size")
    name: str
    strdata: str
    data: list[int]
    _index: int
    _size: int
    def __init__(self, data: str) -> None: ...
    @property
    def index(self) -> int: ...
    @property
    def size(self) -> int: ...
    def reset(self) -> None: ...
    def consume(self) -> None: ...
    def LA(self, offset: int) -> int: ...
    def LT(self, offset: int) -> int: ...
    def mark(self) -> Literal[-1]: ...
    def release(self, marker: int) -> None: ...
    def seek(self, _index: int) -> None: ...
    def getText(self, start: int, stop: int) -> str: ...
