import gdb

inferior_running: bool

def send_process_event_once() -> None: ...
def expect_process(reason: str) -> None: ...
def thread_event(event: gdb.ThreadEvent, reason: str) -> None: ...
def expect_stop(reason: str) -> None: ...
def exec_and_expect_stop(cmd: str, expected_pause: bool = False) -> None: ...

# Map from gdb stop reasons to DAP stop reasons.
stop_reason_map: dict[str, str]
