import ast

def get_sim210(node: ast.IfExp) -> list[tuple[int, int, str]]: ...
def get_sim211(node: ast.IfExp) -> list[tuple[int, int, str]]: ...
def get_sim212(node: ast.IfExp) -> list[tuple[int, int, str]]: ...
