from typing import Final

XK_Hangul: Final = 0xFF31
XK_Hangul_Start: Final = 0xFF32
XK_Hangul_End: Final = 0xFF33
XK_Hangul_Hanja: Final = 0xFF34
XK_Hangul_Jamo: Final = 0xFF35
XK_Hangul_Romaja: Final = 0xFF36
XK_Hangul_Codeinput: Final = 0xFF37
XK_Hangul_Jeonja: Final = 0xFF38
XK_Hangul_Banja: Final = 0xFF39
XK_Hangul_PreHanja: Final = 0xFF3A
XK_Hangul_PostHanja: Final = 0xFF3B
XK_Hangul_SingleCandidate: Final = 0xFF3C
XK_Hangul_MultipleCandidate: Final = 0xFF3D
XK_Hangul_PreviousCandidate: Final = 0xFF3E
XK_Hangul_Special: Final = 0xFF3F
XK_Hangul_switch: Final = 0xFF7E
XK_Hangul_Kiyeog: Final = 0xEA1
XK_Hangul_SsangKiyeog: Final = 0xEA2
XK_Hangul_KiyeogSios: Final = 0xEA3
XK_Hangul_Nieun: Final = 0xEA4
XK_Hangul_NieunJieuj: Final = 0xEA5
XK_Hangul_NieunHieuh: Final = 0xEA6
XK_Hangul_Dikeud: Final = 0xEA7
XK_Hangul_SsangDikeud: Final = 0xEA8
XK_Hangul_Rieul: Final = 0xEA9
XK_Hangul_RieulKiyeog: Final = 0xEAA
XK_Hangul_RieulMieum: Final = 0xEAB
XK_Hangul_RieulPieub: Final = 0xEAC
XK_Hangul_RieulSios: Final = 0xEAD
XK_Hangul_RieulTieut: Final = 0xEAE
XK_Hangul_RieulPhieuf: Final = 0xEAF
XK_Hangul_RieulHieuh: Final = 0xEB0
XK_Hangul_Mieum: Final = 0xEB1
XK_Hangul_Pieub: Final = 0xEB2
XK_Hangul_SsangPieub: Final = 0xEB3
XK_Hangul_PieubSios: Final = 0xEB4
XK_Hangul_Sios: Final = 0xEB5
XK_Hangul_SsangSios: Final = 0xEB6
XK_Hangul_Ieung: Final = 0xEB7
XK_Hangul_Jieuj: Final = 0xEB8
XK_Hangul_SsangJieuj: Final = 0xEB9
XK_Hangul_Cieuc: Final = 0xEBA
XK_Hangul_Khieuq: Final = 0xEBB
XK_Hangul_Tieut: Final = 0xEBC
XK_Hangul_Phieuf: Final = 0xEBD
XK_Hangul_Hieuh: Final = 0xEBE
XK_Hangul_A: Final = 0xEBF
XK_Hangul_AE: Final = 0xEC0
XK_Hangul_YA: Final = 0xEC1
XK_Hangul_YAE: Final = 0xEC2
XK_Hangul_EO: Final = 0xEC3
XK_Hangul_E: Final = 0xEC4
XK_Hangul_YEO: Final = 0xEC5
XK_Hangul_YE: Final = 0xEC6
XK_Hangul_O: Final = 0xEC7
XK_Hangul_WA: Final = 0xEC8
XK_Hangul_WAE: Final = 0xEC9
XK_Hangul_OE: Final = 0xECA
XK_Hangul_YO: Final = 0xECB
XK_Hangul_U: Final = 0xECC
XK_Hangul_WEO: Final = 0xECD
XK_Hangul_WE: Final = 0xECE
XK_Hangul_WI: Final = 0xECF
XK_Hangul_YU: Final = 0xED0
XK_Hangul_EU: Final = 0xED1
XK_Hangul_YI: Final = 0xED2
XK_Hangul_I: Final = 0xED3
XK_Hangul_J_Kiyeog: Final = 0xED4
XK_Hangul_J_SsangKiyeog: Final = 0xED5
XK_Hangul_J_KiyeogSios: Final = 0xED6
XK_Hangul_J_Nieun: Final = 0xED7
XK_Hangul_J_NieunJieuj: Final = 0xED8
XK_Hangul_J_NieunHieuh: Final = 0xED9
XK_Hangul_J_Dikeud: Final = 0xEDA
XK_Hangul_J_Rieul: Final = 0xEDB
XK_Hangul_J_RieulKiyeog: Final = 0xEDC
XK_Hangul_J_RieulMieum: Final = 0xEDD
XK_Hangul_J_RieulPieub: Final = 0xEDE
XK_Hangul_J_RieulSios: Final = 0xEDF
XK_Hangul_J_RieulTieut: Final = 0xEE0
XK_Hangul_J_RieulPhieuf: Final = 0xEE1
XK_Hangul_J_RieulHieuh: Final = 0xEE2
XK_Hangul_J_Mieum: Final = 0xEE3
XK_Hangul_J_Pieub: Final = 0xEE4
XK_Hangul_J_PieubSios: Final = 0xEE5
XK_Hangul_J_Sios: Final = 0xEE6
XK_Hangul_J_SsangSios: Final = 0xEE7
XK_Hangul_J_Ieung: Final = 0xEE8
XK_Hangul_J_Jieuj: Final = 0xEE9
XK_Hangul_J_Cieuc: Final = 0xEEA
XK_Hangul_J_Khieuq: Final = 0xEEB
XK_Hangul_J_Tieut: Final = 0xEEC
XK_Hangul_J_Phieuf: Final = 0xEED
XK_Hangul_J_Hieuh: Final = 0xEEE
XK_Hangul_RieulYeorinHieuh: Final = 0xEEF
XK_Hangul_SunkyeongeumMieum: Final = 0xEF0
XK_Hangul_SunkyeongeumPieub: Final = 0xEF1
XK_Hangul_PanSios: Final = 0xEF2
XK_Hangul_KkogjiDalrinIeung: Final = 0xEF3
XK_Hangul_SunkyeongeumPhieuf: Final = 0xEF4
XK_Hangul_YeorinHieuh: Final = 0xEF5
XK_Hangul_AraeA: Final = 0xEF6
XK_Hangul_AraeAE: Final = 0xEF7
XK_Hangul_J_PanSios: Final = 0xEF8
XK_Hangul_J_KkogjiDalrinIeung: Final = 0xEF9
XK_Hangul_J_YeorinHieuh: Final = 0xEFA
XK_Korean_Won: Final = 0xEFF
