from _typeshed import Incomplete

class Store:
    program: str
    exe: str | None
    environment: Incomplete
    def __init__(self, program: str, environment=None) -> None: ...
    def get(self, server): ...
    def store(self, server, username, secret): ...
    def erase(self, server) -> None: ...
    def list(self): ...
