from _typeshed import Incomplete

from authlib.common.errors import AuthlibBaseError

def extract_header(header_segment: bytes, error_cls: AuthlibBaseError) -> dict[Incomplete, Incomplete]: ...
def extract_segment(segment: bytes, error_cls: AuthlibBaseError, name: str = "payload") -> bytes: ...
def ensure_dict(s: object, structure_name: str) -> dict[Incomplete, Incomplete]: ...
