from typing import Final

__docformat__: Final = "reStructuredText"
document_template: Final[str]

def blahtexml(math_code: str, as_block: bool = False) -> str: ...
def latexml(math_code: str, as_block: bool = False) -> str: ...
def pandoc(math_code: str, as_block: bool = False) -> str: ...
def ttm(math_code: str, as_block: bool = False) -> str: ...
