from unidiff.patch import (
    DEFAULT_ENCODING as DEFAULT_ENCODING,
    LINE_TYPE_ADDED as LINE_TYPE_ADDED,
    LINE_TYPE_CONTEXT as LINE_TYPE_CONTEXT,
    LINE_TYPE_REMOVED as LINE_TYPE_REMOVED,
    Hunk as Hunk,
    PatchedFile as PatchedFile,
    PatchSet as PatchSet,
    UnidiffParseError as UnidiffParseError,
)

VERSION: str
