from authlib.oauth2.rfc6749 import AuthorizationCodeMixin as _AuthorizationCodeMixin

class AuthorizationCodeMixin(_AuthorizationCodeMixin):
    def get_nonce(self) -> str | None: ...
    def get_auth_time(self) -> int | None: ...
    def get_acr(self) -> str: ...
    def get_amr(self) -> list[str]: ...
