from email.message import Message

__all__ = ["encode_7or8bit", "encode_base64", "encode_noop", "encode_quopri"]

def encode_base64(msg: Message) -> None: ...
def encode_quopri(msg: Message) -> None: ...
def encode_7or8bit(msg: Message) -> None: ...
def encode_noop(msg: Message) -> None: ...
