from _typeshed import SupportsItems

def parse_colors(path: str) -> SupportsItems[str, str | tuple[int, int, int]]: ...
def parse_rgb_txt_file(path: str) -> SupportsItems[str, str | tuple[int, int, int]]: ...
def parse_json_color_file(path: str) -> dict[str, str]: ...
def sanitize_color_palette(colorpalette: SupportsItems[str, str | tuple[int, int, int]]) -> dict[str, tuple[int, int, int]]: ...
