from typing import Literal, overload

@overload
def wildcard_match(pattern: None, text: str, case_insensitive: bool = True) -> Literal[False]: ...
@overload
def wildcard_match(pattern: str, text: None, case_insensitive: bool = True) -> Literal[False]: ...
@overload
def wildcard_match(pattern: str, text: str, case_insensitive: bool = True) -> bool: ...
