from typing import Final

from .classic import deprecated as deprecated
from .params import deprecated_params as deprecated_params

__version__: Final[str]
__author__: Final[str]
__date__: Final[str]
__credits__: Final[str]
